from unittest import skip

from django.urls import reverse

from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.search import search_model_document
from mayan.apps.documents.tests.mixins.document_mixins import (
    DocumentTestMixin, DocumentViewTestMixin
)
from mayan.apps.testing.tests.base import GenericViewTestCase

from ..literals import (
    MATCH_ALL_FIELD_CHOICES, MATCH_ALL_FIELD_NAME, MATCH_ALL_VALUES,
    QUERY_PARAMETER_ANY_FIELD, SEARCH_MODEL_NAME_KWARG
)
from ..permissions import permission_search_tools

from .literals import TEST_SEARCH_OBJECT_TERM
from .mixins.base import SearchTestMixin, TestSearchObjectSimpleTestMixin
from .mixins.view_mixins import SearchToolsViewTestMixin, SearchViewTestMixin


class SearchAdvancedViewTestCase(
    DocumentTestMixin, SearchViewTestMixin, TestSearchObjectSimpleTestMixin,
    SearchTestMixin, GenericViewTestCase
):
    auto_test_search_objects_create = False
    auto_upload_test_document = False

    def test_advanced_search_past_first_page_view_with_access(self):
        self._create_test_document_stubs(count=4)

        for test_document in self._test_documents:
            self.grant_access(
                obj=test_document, permission=permission_document_view
            )

        # Make sure all documents are returned by the search.
        queryset = self._test_search_backend.search(
            search_model=search_model_document,
            query={
                'label': '*{}'.format(TEST_SEARCH_OBJECT_TERM)
            },
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 4)

        self._clear_events()

        with self.settings(VIEWS_PAGINATE_BY=2):
            # Functional test for the first page of advanced results.
            response = self._request_search_results_view(
                data={
                    'label': '*{}'.format(TEST_SEARCH_OBJECT_TERM)
                }, kwargs={
                    SEARCH_MODEL_NAME_KWARG: search_model_document.full_name
                }
            )

            # Total (1 - 2 out of 4) (Page 1 of 2)
            # 4 results total, 2 pages, current page is 1,
            # object in this page: 2
            self.assertEqual(response.status_code, 200)
            self.assertEqual(
                response.context['paginator'].object_list.count(), 4
            )
            self.assertEqual(response.context['paginator'].num_pages, 2)
            self.assertEqual(response.context['page_obj'].number, 1)
            self.assertEqual(
                response.context['page_obj'].object_list.count(), 2
            )

            events = self._get_test_events()
            self.assertEqual(events.count(), 0)

            self._clear_events()

            # Functional test for the second page of advanced results.
            response = self._request_search_results_view(
                data={
                    'label': '*{}'.format(TEST_SEARCH_OBJECT_TERM),
                    'page': 2
                }, kwargs={
                    SEARCH_MODEL_NAME_KWARG: search_model_document.full_name
                }
            )
            # Total (3 - 4 out of 4) (Page 2 of 2)
            # 4 results total, 2 pages, current page is 1,
            # object in this page: 2
            self.assertEqual(response.status_code, 200)
            self.assertEqual(
                response.context['paginator'].object_list.count(), 4
            )
            self.assertEqual(response.context['paginator'].num_pages, 2)
            self.assertEqual(response.context['page_obj'].number, 2)
            self.assertEqual(
                response.context['page_obj'].object_list.count(), 2
            )

            events = self._get_test_events()
            self.assertEqual(events.count(), 0)


class FilterViewMixinTestCase(
    DocumentTestMixin, DocumentViewTestMixin, SearchTestMixin,
    SearchViewTestMixin, TestSearchObjectSimpleTestMixin, GenericViewTestCase
):
    auto_test_search_objects_create = False
    auto_upload_test_document = False

    def test_document_list_filter_view_with_access(self):
        self._create_test_document_stubs(count=4)

        for test_document in self._test_documents:
            self.grant_access(
                obj=test_document, permission=permission_document_view
            )

        self._clear_events()

        response = self._request_test_document_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self._test_document.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self._clear_events()

        response = self._request_test_document_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[0].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[1].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[2].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[3].label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self._clear_events()

        response = self._request_test_document_list_view(
            data={'filter_q': '*stub_3'}
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_documents[0].label
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_documents[1].label
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_documents[2].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[3].label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class SearchFilterViewTestCase(
    DocumentTestMixin, SearchTestMixin, SearchViewTestMixin,
    TestSearchObjectSimpleTestMixin, GenericViewTestCase
):
    auto_test_search_objects_create = False
    auto_upload_test_document = False

    def test_basic_search_filter_list_with_access(self):
        self._create_test_document_stubs(count=4)

        for test_document in self._test_documents:
            self.grant_access(
                obj=test_document, permission=permission_document_view
            )

        self._clear_events()

        response = self._request_search_results_view(
            data={
                'q': '*{}'.format(TEST_SEARCH_OBJECT_TERM)
            }, kwargs={
                SEARCH_MODEL_NAME_KWARG: search_model_document.full_name
            }
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[0].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[1].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[2].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[3].label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        self._clear_events()

        response = self._request_search_results_view(
            data={
                'q': TEST_SEARCH_OBJECT_TERM,
                'filter_q': '*stub_3',
            }, kwargs={
                SEARCH_MODEL_NAME_KWARG: search_model_document.full_name
            }
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_documents[0].label
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_documents[1].label
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_documents[2].label
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_documents[3].label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class SearchViewTestCase(
    SearchViewTestMixin, TestSearchObjectSimpleTestMixin, SearchTestMixin,
    GenericViewTestCase
):
    auto_test_search_objects_create = False

    def test_search_simple_get_view_no_permission(self):
        self._clear_events()

        response = self._request_search_simple_get_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_advanced_get_view_no_permission(self):
        self._clear_events()

        response = self._request_search_advanced_get_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_again_advanced_view_no_permission(self):
        self._clear_events()

        response = self._request_search_again_view(
            query={'label': 'test'}
        )
        expected_url = '{}?{}'.format(
            reverse(
                viewname='search:search_advanced', kwargs={
                    'search_model_pk': self._test_search_model.full_name
                }
            ), 'label=test'
        )
        self.assertRedirects(
            response=response, expected_url=expected_url, status_code=302,
            target_status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_again_simple_view_no_permission(self):
        self._clear_events()

        response = self._request_search_again_view(
            query={'q': 'test'}
        )
        expected_url = '{}?{}'.format(
            reverse(
                viewname='search:search_simple', kwargs={
                    'search_model_pk': self._test_search_model.full_name
                }
            ), 'q=test'
        )
        self.assertRedirects(
            response=response, expected_url=expected_url, status_code=302,
            target_status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_again_advanced_match_all_view_no_permission(self):
        self._clear_events()

        response = self._request_search_again_view(
            follow=True, query={
                'label': 'test', MATCH_ALL_FIELD_NAME: MATCH_ALL_VALUES[0]
            }
        )

        self.assertContains(
            html=True, response=response, status_code=200,
            text='<input type="radio" name="{name}" value="{value}" id="id_{id}_0">'.format(
                id=MATCH_ALL_FIELD_NAME, name=MATCH_ALL_FIELD_NAME,
                value=MATCH_ALL_FIELD_CHOICES[0][0]
            )
        )
        self.assertContains(
            html=True, response=response, status_code=200,
            text='<input type="radio" name="{name}" value="{value}" id="id_{id}_1">'.format(
                id=MATCH_ALL_FIELD_NAME, name=MATCH_ALL_FIELD_NAME,
                value=MATCH_ALL_FIELD_CHOICES[1][0]
            )
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class SearchResultViewTestCase(
    SearchViewTestMixin, TestSearchObjectSimpleTestMixin, SearchTestMixin,
    GenericViewTestCase
):
    def test_result_view_any_field_no_value(self):
        self._clear_events()

        response = self._request_search_results_view(
            data={
                'q': '',
                '_{}'.format(
                    SEARCH_MODEL_NAME_KWARG
                ): self._test_search_model.full_name
            }
        )
        self.assertNotContains(
            response=response, status_code=200,
            text='Total: 1'
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_result_view_any_field_value(self):
        self._clear_events()

        response = self._request_search_results_view(
            data={
                'q': self._test_object.uuid,
                '_{}'.format(
                    SEARCH_MODEL_NAME_KWARG
                ): self._test_search_model.full_name
            }
        )
        self.assertContains(
            response=response, status_code=200,
            text='Total: 1'
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_result_advanced_view(self):
        self._clear_events()

        response = self._request_search_results_view(
            data={
                'uuid': self._test_object.uuid,
                '_{}'.format(
                    SEARCH_MODEL_NAME_KWARG
                ): self._test_search_model.full_name
            }
        )
        self.assertContains(
            response=response, status_code=200,
            text='Total: 1'
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_result_scoped_search_view(self):
        self._clear_events()

        response = self._request_search_results_view(
            data={
                '__0_uuid': self._test_object.uuid,
                '__result': '0',
                '_{}'.format(
                    SEARCH_MODEL_NAME_KWARG
                ): self._test_search_model.full_name
            }
        )
        self.assertContains(
            response=response, status_code=200,
            text='Total: 1'
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class SearchToolsViewTestCase(
    SearchToolsViewTestMixin, TestSearchObjectSimpleTestMixin,
    SearchTestMixin, GenericViewTestCase
):
    @skip(reason='Test with a backend that supports reindexing.')
    def test_search_backend_reindex_view_no_permission(self):
        self._test_search_backend.reset(
            search_model=self._test_search_model
        )

        self._clear_events()

        response = self._request_search_backend_reindex_view()
        self.assertEqual(response.status_code, 403)

        queryset = self._test_search_backend.search(
            search_model=self._test_search_model,
            query={QUERY_PARAMETER_ANY_FIELD: str(self._test_object.uuid)},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    @skip(reason='Test with a backend that supports reindexing.')
    def test_search_backend_reindex_view_with_permission(self):
        self._test_search_backend.reset(
            search_model=self._test_search_model
        )

        self.grant_permission(permission=permission_search_tools)

        self._clear_events()

        response = self._request_search_backend_reindex_view()
        self.assertEqual(response.status_code, 302)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

        queryset = self._test_search_backend.search(
            search_model=self._test_search_model,
            query={QUERY_PARAMETER_ANY_FIELD: str(self._test_object.uuid)},
            user=self._test_case_user
        )
        self.assertNotEqual(queryset.count(), 0)



from django.core import management

from ...classes import Setting


class Command(management.BaseCommand):
    help = 'Rollback the configuration file to the last valid version.'

    def handle(self, *args, **options):
        try:
            Setting.revert_configuration()
        except Exception as exception:
            self.stderr.write(
                msg=self.style.NOTICE(
                    str(exception)
                )
            )
            exit(1)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.rest_api import serializers
from mayan.apps.rest_api.relations import MultiKwargHyperlinkedIdentityField

from ..models.document_type_models import DocumentType, DocumentTypeFilename


class DocumentTypeQuickLabelSerializer(serializers.ModelSerializer):
    document_type_url = serializers.HyperlinkedIdentityField(
        label=_('Document type URL'), lookup_url_kwarg='document_type_id',
        view_name='rest_api:documenttype-detail'
    )
    url = MultiKwargHyperlinkedIdentityField(
        label=_('URL'), view_kwargs=(
            {
                'lookup_field': 'document_type_id',
                'lookup_url_kwarg': 'document_type_id'
            },
            {
                'lookup_field': 'pk',
                'lookup_url_kwarg': 'document_type_quick_label_id'
            }
        ), view_name='rest_api:documenttype-quicklabel-detail'
    )

    class Meta:
        fields = ('document_type_url', 'enabled', 'filename', 'id', 'url')
        model = DocumentTypeFilename
        read_only_fields = ('document_type_url', 'id', 'url')


class DocumentTypeSerializer(serializers.HyperlinkedModelSerializer):
    quick_label_list_url = serializers.HyperlinkedIdentityField(
        label=_('Quick label list URL'), lookup_url_kwarg='document_type_id',
        view_name='rest_api:documenttype-quicklabel-list'
    )

    class Meta:
        extra_kwargs = {
            'url': {
                'label': _('URL'),
                'lookup_url_kwarg': 'document_type_id',
                'view_name': 'rest_api:documenttype-detail'
            }
        }
        fields = (
            'delete_time_period', 'delete_time_unit',
            'document_stub_expiration_interval',
            'document_stub_pruning_enabled', 'filename_generator_backend',
            'filename_generator_backend_arguments', 'id', 'label',
            'quick_label_list_url', 'trash_time_period', 'trash_time_unit',
            'url'
        )
        model = DocumentType
        read_only_fields = ('id', 'quick_label_list_url', 'url')



from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from .events import event_acl_edited


class AccessControlListBusinessLogicMixin:
    def get_inherited_permissions(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        return AccessControlList.objects.get_inherited_permissions(
            obj=self.content_object, role=self.role
        )

    def get_permission_count(self):
        """
        Return the numeric count of permissions that have this role
        has granted. The count is filtered by access.
        """
        return self.permissions.count()
    get_permission_count.short_description = _('Permission count')

    def permissions_add(self, queryset, user):
        for obj in queryset:
            self.permissions.add(obj)

        event_acl_edited.commit(
            action_object=self.content_object,
            actor=user, target=self
        )

    def permissions_remove(self, queryset, user):
        for obj in queryset:
            self.permissions.remove(obj)

        event_acl_edited.commit(
            action_object=self.content_object,
            actor=user or self._event_actor, target=self
        )



from django.utils.translation import ugettext_lazy as _

DATABASE_MINIMUM_ID = 1

DJANGO_POSITIVE_INTEGER_FIELD_MAX_VALUE = 2147483647

DJANGO_SQLITE_BACKEND = 'django.db.backends.sqlite3'

MESSAGE_SQLITE_WARNING = _(
    'Your database backend is set to use SQLite. SQLite should only be used '
    'for development and testing, not for production.'
)



from ..models import FavoriteDocument
from ..permissions import permission_document_view

from .base import GenericDocumentViewTestCase
from .mixins.favorite_document_mixins import (
    FavoriteDocumentTestMixin, FavoriteDocumentsViewTestMixin
)


class FavoriteDocumentsTestCase(
    FavoriteDocumentTestMixin, FavoriteDocumentsViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()

    def test_document_favorite_add_view_no_permission(self):
        favorite_document_count = FavoriteDocument.valid.count()

        self._clear_events()

        response = self._request_test_document_favorite_add_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_favorite_add_view_with_access(self):
        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_document_favorite_add_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_add_view_with_access(self):
        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_favorite_add_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_favorite_list_view_no_permission(self):
        self._test_document_favorite_add()

        self._clear_events()

        response = self._request_test_document_favorites_list_view()
        self.assertNotContains(
            response=response, text=self._test_document.label, status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_favorite_list_view_with_access(self):
        self._test_document_favorite_add()
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_document_favorites_list_view()
        self.assertContains(
            response=response, text=self._test_document.label, status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_list_view_with_access(self):
        self._test_document_favorite_add()

        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_favorites_list_view()

        self.assertNotContains(
            response=response, text=self._test_document.label, status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_favorite_remove_view_no_permission(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self._clear_events()

        response = self._request_test_document_favorite_remove_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_favorite_remove_view_with_access(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_document_favorite_remove_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_remove_view_with_access(self):
        self._test_document_favorite_add()

        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._test_document.delete()

        favorite_document_count = FavoriteDocument.valid.count()

        self._clear_events()

        response = self._request_test_document_favorite_remove_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('document_indexing', '0006_auto_20150729_0144'),
    ]

    operations = [
        migrations.AlterField(
            model_name='index',
            name='slug',
            field=models.SlugField(
                default='', max_length=128, help_text='This values will be '
                'used by other apps to reference this index.', unique=True,
                verbose_name='Slug'
            ),
            preserve_default=False,
        ),
    ]



from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0010_auto_20150704_0054'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttype',
            name='delete_time_period',
            field=models.PositiveIntegerField(
                default=30, verbose_name='Delete time period'
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='documenttype',
            name='delete_time_unit',
            field=models.CharField(
                default='days', max_length=8, verbose_name='Delete time unit',
                choices=[
                    ('days', 'Days'), ('hours', 'Hours'),
                    ('minutes', 'Minutes'), ('seconds', 'Seconds')
                ]
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='documenttype',
            name='trash_time_period',
            field=models.PositiveIntegerField(
                null=True, verbose_name='Trash time period', blank=True
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='documenttype',
            name='trash_time_unit',
            field=models.CharField(
                blank=True, max_length=8, null=True,
                verbose_name='Trash time unit', choices=[
                    ('days', 'Days'), ('hours', 'Hours'),
                    ('minutes', 'Minutes'), ('seconds', 'Seconds')
                ]
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='document',
            name='deleted_date_time',
            field=models.DateTimeField(
                verbose_name='Date and time trashed', blank=True
            ),
            preserve_default=True,
        ),
    ]



from django.apps import apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.template import Library
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import get_template
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from ..literals import COMMENT_APP_TEMPLATE_CACHE_DISABLE

app_templates_cache = {}
register = Library()


@register.simple_tag(takes_context=True)
def appearance_app_templates(context, template_name):
    """
    Fetch the app templates for the requested `template_name`, render it with
    the current `request` from the `context`, and cache it for future use
    unless the template has the no caching comment.
    """
    result = []

    for app in apps.get_app_configs():
        template_id = '{}.{}'.format(app.label, template_name)
        if template_id not in app_templates_cache or settings.DEBUG:
            try:
                app_template = get_template(
                    '{}/app/{}.html'.format(app.label, template_name)
                )
                app_template_output = app_template.render(
                    context=context.flatten(), request=context.get('request')
                )

                if COMMENT_APP_TEMPLATE_CACHE_DISABLE not in app_template.template.source:
                    app_templates_cache[template_id] = app_template_output
            except TemplateDoesNotExist:
                """
                Non fatal just means that the app did not defined an app
                template of this name and purpose.
                """
                app_templates_cache[template_id] = ''
                app_template_output = ''
        else:
            app_template_output = app_templates_cache[template_id]

        result.append(app_template_output)

    return mark_safe(
        s=' '.join(result)
    )


@register.filter
def appearance_form_get_visile_fields_map(form):
    field_map = {
        field.name: field for field in form.visible_fields()
    }
    return field_map


@register.filter
def appearance_get_choice_value(field):
    try:
        return dict(field.field.choices)[
            field.value()
        ]
    except TypeError:
        return ', '.join(
            [
                subwidget.data['label'] for subwidget in field.subwidgets if subwidget.data['selected']
            ]
        )
    except KeyError:
        return _('None')


@register.filter
def appearance_get_form_media_js(form):
    return [
        form.media.absolute_path(path) for path in form.media._js
    ]


@register.simple_tag
def appearance_get_icon(icon_path, **kwargs):
    clean_kwargs = {}

    for key, value in kwargs.items():
        if '__' in key:
            subdictionary = clean_kwargs
            parts = key.split('__')
            for part in parts:
                subdictionary.setdefault(
                    part, {}
                )
                dictionary_pointer = subdictionary
                subdictionary = subdictionary[part]

            dictionary_pointer[part] = value
        else:
            clean_kwargs[key] = value

    return import_string(dotted_path=icon_path).render(**clean_kwargs)


@register.simple_tag
def appearance_get_user_theme_stylesheet(user):
    User = get_user_model()

    if user and user.is_authenticated:
        try:
            theme = user.theme_settings.theme
        except User.theme_settings.RelatedObjectDoesNotExist:
            # User had a setting assigned which was later deleted.
            return ''
        else:
            if theme:
                return user.theme_settings.theme.stylesheet

    return ''


@register.simple_tag
def appearance_icon_render(icon, enable_shadow=False):
    return icon.render(
        extra_context={'enable_shadow': enable_shadow}
    )


@register.filter
def appearance_object_list_count(object_list):
    try:
        return object_list.count()
    except TypeError:
        return len(object_list)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0064_auto_20201012_0544'),
    ]

    operations = [
        migrations.AddField(
            model_name='documentversion',
            name='active',
            field=models.BooleanField(
                default=True, help_text='Determines the active version '
                'of the document.', verbose_name='Active'
            ),
        ),
    ]



from django.apps import apps
from django.core.exceptions import PermissionDenied

from mayan.apps.navigation.column_widgets import SourceColumnWidget

from .permissions import permission_document_metadata_view


class DocumentMetadataWidget(SourceColumnWidget):
    """
    A widget that displays the metadata for the given document.
    """
    template_name = 'metadata/document_metadata_widget.html'

    def get_extra_context(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        try:
            AccessControlList.objects.check_access(
                obj=self.value,
                permissions=(permission_document_metadata_view,),
                user=self.request.user
            )
        except PermissionDenied:
            queryset = self.value.metadata.none()
        else:
            queryset = self.value.get_metadata(
                permission=permission_document_metadata_view,
                user=self.request.user
            )

        return {'queryset': queryset}



from django.conf import settings
from django.db import models
from django.shortcuts import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.validators import validate_internal_name
from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.documents.models.document_models import Document
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import (
    EventManagerMethodAfter, EventManagerSave
)

from .events import (
    event_signature_capture_created, event_signature_capture_deleted,
    event_signature_capture_edited
)
from .managers import ValidSignatureCaptureManager
from .model_mixins import SignatureCaptureBusinessLogicMixin


class SignatureCapture(
    ExtraDataModelMixin, SignatureCaptureBusinessLogicMixin, models.Model
):
    document = models.ForeignKey(
        on_delete=models.CASCADE, related_name='signature_captures',
        to=Document, verbose_name=_('Document')
    )
    data = models.TextField(
        blank=True, help_text=_(
            'Data representing the handwritten signature.'
        ), verbose_name=_('Signature capture data')
    )
    svg = models.TextField(
        blank=True, help_text=_(
            'Vector representation of the handwritten signature.'
        ), verbose_name=_('SVG signature capture data')
    )
    text = models.CharField(
        help_text=_('Print version of the captured signature.'),
        max_length=224, verbose_name=_('Text')
    )
    user = models.ForeignKey(
        on_delete=models.CASCADE, related_name='signature_captures',
        to=settings.AUTH_USER_MODEL, verbose_name=_('User')
    )
    date_time_created = models.DateTimeField(
        auto_now_add=True, db_index=True,
        verbose_name=_('Date and time created')
    )
    date_time_edited = models.DateTimeField(
        auto_now=True, db_index=True,
        verbose_name=_('Date and time edited')
    )
    internal_name = models.CharField(
        db_index=True, help_text=_(
            'This value will be used when referencing this signature '
            'capture in relationship to the document. Can only contain '
            'letters, numbers, and underscores.'
        ), max_length=255, validators=[validate_internal_name],
        verbose_name=_('Internal name')
    )

    objects = models.Manager()
    valid = ValidSignatureCaptureManager()

    class Meta:
        ordering = ('-date_time_created',)
        unique_together = ('document', 'internal_name')
        verbose_name = _('Signature capture')
        verbose_name_plural = _('Signature captures')

    def __str__(self):
        return '{} - {}'.format(
            self.text, self.get_date_time_created()
        )

    @method_event(
        action_object='self',
        event=event_signature_capture_deleted,
        event_manager_class=EventManagerMethodAfter,
        target='document'
    )
    def delete(self, *args, **kwargs):
        self.cache_partition.delete()
        return super().delete(*args, **kwargs)

    def get_absolute_url(self):
        return reverse(
            viewname='signature_captures:signature_capture_detail', kwargs={
                'signature_capture_id': self.pk
            }
        )

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'actor': 'user',
            'action_object': 'document',
            'event': event_signature_capture_created,
            'target': 'self'
        },
        edited={
            'action_object': 'document',
            'event': event_signature_capture_edited,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)



from django.apps import apps

from .literals import STORAGE_NAME_ASSETS_CACHE
from .settings import setting_asset_cache_maximum_size


def handler_create_asset_cache(sender, **kwargs):
    Cache = apps.get_model(app_label='file_caching', model_name='Cache')

    Cache.objects.update_or_create(
        defaults={
            'maximum_size': setting_asset_cache_maximum_size.value,
        }, defined_storage_name=STORAGE_NAME_ASSETS_CACHE,
    )



TEST_INDEX_TEMPLATE_DOCUMENT_LABEL_EXPRESSION = '{{ document.label }}'
TEST_INDEX_TEMPLATE_DOCUMENT_DESCRIPTION_EXPRESSION = '{{ document.description }}'
TEST_INDEX_TEMPLATE_DOCUMENT_TYPE_EXPRESSION = '{{ document.document_type.label }}'

TEST_INDEX_TEMPLATE_LABEL = 'test label'
TEST_INDEX_TEMPLATE_LABEL_EDITED = 'test edited label'
TEST_INDEX_TEMPLATE_SLUG = 'test_slug'

TEST_INDEX_TEMPLATE_NODE_EXPRESSION = '{{ document.pk }}'
TEST_INDEX_TEMPLATE_NODE_EXPRESSION_EDITED = '{{ document.description }}'



from django.db.models import Q

from ...models.workflow_transition_models import WorkflowTransitionField

from ..literals import (
    TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_HELP_TEXT,
    TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_LABEL,
    TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_LABEL_EDITED,
    TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_NAME,
    TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_TYPE
)

from .workflow_template_transition_mixins import WorkflowTemplateTransitionTestMixin


class WorkflowTemplateTransitionFieldTestMixin(
    WorkflowTemplateTransitionTestMixin
):
    def _create_test_workflow_template_transition_field(self, extra_data=None):
        kwargs = {
            'field_type': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_TYPE,
            'help_text': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_HELP_TEXT,
            'label': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_LABEL,
            'name': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_NAME
        }
        kwargs.update(
            extra_data or {}
        )

        self._test_workflow_template_transition_field = self._test_workflow_template_transition.fields.create(
            **kwargs
        )


class WorkflowTemplateTransitionFieldAPIViewTestMixin(
    WorkflowTemplateTransitionFieldTestMixin
):
    def _request_test_workflow_template_transition_field_create_api_view(self):
        pk_list = list(
            WorkflowTransitionField.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='rest_api:workflow-template-transition-field-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_transition_id': self._test_workflow_template_transition.pk,
            }, data={
                'field_type': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_TYPE,
                'help_text': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_HELP_TEXT,
                'label': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_LABEL,
                'name': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_NAME
            }
        )

        try:
            self._test_workflow_template_transition_field = WorkflowTransitionField.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowTransitionField.DoesNotExist:
            self._test_workflow_template_transition_field = None

        return response

    def _request_test_workflow_template_transition_field_delete_api_view(self):
        return self.delete(
            viewname='rest_api:workflow-template-transition-field-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_transition_id': self._test_workflow_template_transition.pk,
                'workflow_template_transition_field_id': self._test_workflow_template_transition_field.pk
            }
        )

    def _request_test_workflow_template_transition_field_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-transition-field-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_transition_id': self._test_workflow_template_transition.pk,
                'workflow_template_transition_field_id': self._test_workflow_template_transition_field.pk
            }
        )

    def _request_test_workflow_template_transition_field_edit_via_patch_api_view(self):
        return self.patch(
            viewname='rest_api:workflow-template-transition-field-detail',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_transition_id': self._test_workflow_template_transition.pk,
                'workflow_template_transition_field_id': self._test_workflow_template_transition_field.pk
            }, data={
                'label': '{} edited'.format(
                    self._test_workflow_template_transition_field
                )
            }
        )

    def _request_test_workflow_template_transition_field_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-transition-field-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_transition_id': self._test_workflow_template_transition.pk
            }
        )


class WorkflowTemplateTransitionFieldViewTestMixin(
    WorkflowTemplateTransitionFieldTestMixin
):
    def _request_workflow_template_transition_field_create_view(self):
        pk_list = list(
            WorkflowTransitionField.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='document_states:workflow_template_transition_field_create',
            kwargs={
                'workflow_template_transition_id': self._test_workflow_template_transition.pk
            }, data={
                'field_type': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_TYPE,
                'help_text': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_HELP_TEXT,
                'label': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_LABEL,
                'name': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_NAME
            }
        )

        try:
            self._test_workflow_template_transition_field = WorkflowTransitionField.objects.get(
                ~Q(pk__in=pk_list)
            )
        except WorkflowTransitionField.DoesNotExist:
            self._test_workflow_template_transition_field = None

        return response

    def _request_workflow_template_transition_field_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_field_delete',
            kwargs={
                'workflow_template_transition_field_id': self._test_workflow_template_transition_field.pk
            }
        )

    def _request_workflow_template_transition_field_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_field_edit',
            kwargs={
                'workflow_template_transition_field_id': self._test_workflow_template_transition_field.pk
            }, data={
                'field_type': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_TYPE,
                'help_text': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_HELP_TEXT,
                'label': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_LABEL_EDITED,
                'name': TEST_WORKFLOW_TEMPLATE_TRANSITION_FIELD_NAME
            }
        )

    def _request_test_workflow_template_transition_field_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_transition_field_list',
            kwargs={
                'workflow_template_transition_id': self._test_workflow_template_transition.pk
            }
        )



from mayan.apps.django_gpg.tests.mixins import KeyTestMixin
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.documents.tests.literals import TEST_FILE_SMALL_PATH

from ..permissions import permission_document_file_signature_view

from .mixins import DetachedSignatureTestMixin, SignatureViewTestMixin


class SignaturesViewTestCase(
    KeyTestMixin, DetachedSignatureTestMixin, SignatureViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def test_signature_detail_view_no_permission(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()
        self._upload_test_detached_signature()

        self._clear_events()

        response = self._request_test_document_file_signature_detail_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_detail_view_with_access(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self._create_test_key_public()
        self._upload_test_detached_signature()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_signature_view
        )

        self._clear_events()

        response = self._request_test_document_file_signature_detail_view()
        self.assertContains(
            response=response, text=self._test_signature.signature_id,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_detail_view_with_access(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self._create_test_key_public()
        self._upload_test_detached_signature()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_signature_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_signature_detail_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_list_view_no_permission(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self._upload_test_detached_signature()

        self._clear_events()

        response = self._request_test_document_file_signature_list_view(
            document=self._test_document
        )
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_list_view_with_access(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self._upload_test_detached_signature()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_signature_view
        )

        self._clear_events()

        response = self._request_test_document_file_signature_list_view(
            document=self._test_document
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['object_list'].count(), 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_list_view_with_access(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self._upload_test_detached_signature()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_signature_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_signature_list_view(
            document=self._test_document
        )
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from rest_framework import status

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import event_workflow_template_edited
from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)

from .mixins.workflow_template_state_action_mixins import WorkflowTemplateStateActionAPIViewTestMixin


class WorkflowTemplateStateActionsAPIViewTestCase(
    WorkflowTemplateStateActionAPIViewTestMixin, BaseAPITestCase
):
    def test_workflow_template_state_action_create_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_workflow_template_state_action_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_workflow_template.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_state.actions.count(), 0
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self._test_workflow_template.refresh_from_db()

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_state_action_delete_api_view_no_permission(self):
        self._create_test_workflow_template_state_action()

        self._clear_events()

        response = self._request_test_workflow_template_state_action_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_workflow_template.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_state.actions.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_delete_api_view_with_access(self):
        self._create_test_workflow_template_state_action()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self._test_workflow_template.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_state.actions.count(), 0
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_state_action_detail_api_view_no_permission(self):
        self._create_test_workflow_template_state_action()

        self._clear_events()

        response = self._request_test_workflow_template_state_action_detail_api_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_detail_api_view_with_access(self):
        self._create_test_workflow_template_state_action()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self._test_workflow_template_state_action.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_edit_api_view_via_patch_no_permission(self):
        self._create_test_workflow_template_state_action()

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_workflow_template_state_action.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_edit_api_view_via_patch_with_access(self):
        self._create_test_workflow_template_state_action()

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_patch_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_workflow_template_state_action.refresh_from_db()
        self.assertNotEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].action_object, self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_state_action_edit_api_view_via_put_no_permission(self):
        self._create_test_workflow_template_state_action()

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_workflow_template_state_action.refresh_from_db()
        self.assertEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_edit_api_view_via_put_with_access(self):
        self._create_test_workflow_template_state_action()

        test_workflow_template_state_action_label = self._test_workflow_template_state_action.label

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_edit_put_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_workflow_template_state_action.refresh_from_db()
        self.assertNotEqual(
            self._test_workflow_template_state_action.label,
            test_workflow_template_state_action_label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].action_object,
            self._test_workflow_template_state_action
        )
        self.assertEqual(events[0].target, self._test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_state_action_list_api_view_no_permission(self):
        self._create_test_workflow_template_state_action()

        self._clear_events()

        response = self._request_test_workflow_template_state_action_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_state_action_list_api_view_with_access(self):
        self._create_test_workflow_template_state_action()

        self.grant_access(
            obj=self._test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_state_action_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['id'],
            self._test_workflow_template_state_action.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



import json


class StatisticResultBusinessLogicMixin:
    def get_data(self):
        return json.loads(s=self.serialize_data)

    def store_data(self, data):
        self.serialize_data = json.dumps(obj=data)
        self.save()



class DynamicFormBackendMixin:
    """
    The fields attribute is a list of dictionaries with the format:
    {
        'name': ''  # Field internal name
        'label': ''  # Label to show to users
        'initial': ''  # Field initial value
        'default': ''  # Default value.
    }
    """
    form_fieldset_exclude_list = ('backend_data',)

    @classmethod
    def get_form_field_order(cls):
        return getattr(
            cls, 'form_field_order', ()
        )

    @classmethod
    def get_form_field_widgets(cls):
        return getattr(
            cls, 'form_field_widgets', {}
        )

    @classmethod
    def get_form_fields(cls):
        return getattr(
            cls, 'form_fields', {}
        )

    @classmethod
    def get_form_fieldset_exclude_list(cls):
        return getattr(
            cls, 'form_fieldset_exclude_list', ()
        )

    @classmethod
    def get_form_fieldsets(cls):
        return getattr(
            cls, 'form_fieldsets', ()
        )

    @classmethod
    def get_form_media(cls):
        return getattr(
            cls, 'form_media', {}
        )

    @classmethod
    def get_form_schema(cls, *args, **kwargs):
        form_fields = cls.get_form_fields(**kwargs)

        result = {
            'fields': form_fields,
            'fieldset_exclude_list': cls.get_form_fieldset_exclude_list(**kwargs),
            'fieldsets': cls.get_form_fieldsets(**kwargs),
            'media': cls.get_form_media(),
            'widgets': cls.get_form_field_widgets(**kwargs)
        }

        form_field_order = cls.get_form_field_order(**kwargs) or tuple(
            form_fields.keys()
        )

        result['field_order'] = form_field_order

        return result



TEST_CASE_ACTION_NAME_SOURCE_CREATE = 'source-create'

TEST_CASE_INTERFACE_NAME_MODEL = 'model'
TEST_CASE_INTERFACE_NAME_REST_API = 'rest-api'
TEST_CASE_INTERFACE_NAME_VIEW = 'view'

TEST_SOURCE_ACTION_CONFIRM_FALSE_NAME = 'action_test_no_confirm'
TEST_SOURCE_ACTION_CONFIRM_TRUE_NAME = 'action_test_confirm'

TEST_SOURCE_BACKEND_PATH_BASE = 'mayan.apps.sources.tests.source_backends.SourceBackendTest'
TEST_SOURCE_BACKEND_PATH_DOCUMENT_BASIC = 'mayan.apps.sources.tests.source_backends.SourceBackendDocumentUploadTest'

TEST_SOURCE_LABEL = 'test source'
TEST_SOURCE_LABEL_EDITED = 'test source edited'

TEST_SOURCE_METADATA_KEY = 'test_source_metadata_key'
TEST_SOURCE_METADATA_VALUE = 'test_source_metadata_value'



from django.contrib import messages
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.views.forms import DynamicForm
from mayan.apps.views.generics import FormView, SingleObjectListView
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms import WorkflowInstanceTransitionSelectForm
from ..icons import (
    icon_workflow_instance_detail, icon_workflow_instance_list,
    icon_workflow_instance_transition,
    icon_workflow_instance_transition_select, icon_workflow_template_list
)
from ..links import link_workflow_instance_transition
from ..literals import FIELD_TYPE_MAPPING, WIDGET_CLASS_MAPPING
from ..models import WorkflowInstance
from ..permissions import (
    permission_workflow_instance_transition, permission_workflow_template_view
)


class WorkflowInstanceListView(ExternalObjectViewMixin, SingleObjectListView):
    external_object_permission = permission_workflow_template_view
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    object_permission = permission_workflow_template_view
    view_icon = icon_workflow_instance_list

    def get_extra_context(self):
        return {
            'hide_link': True,
            'no_results_icon': icon_workflow_template_list,
            'no_results_text': _(
                'Assign workflows to the document type of this document '
                'to have this document execute those workflows. '
            ),
            'no_results_title': _(
                'There are no workflows for this document'
            ),
            'object': self.external_object,
            'title': _(
                'Workflows for document: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.workflows.all()


class WorkflowInstanceDetailView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_permission = permission_workflow_template_view
    external_object_pk_url_kwarg = 'workflow_instance_id'
    object_permission = permission_workflow_template_view
    view_icon = icon_workflow_instance_detail

    def get_extra_context(self):
        return {
            'hide_object': True,
            'navigation_object_list': ('object', 'workflow_instance'),
            'no_results_icon': icon_workflow_instance_detail,
            'no_results_main_link': link_workflow_instance_transition.resolve(
                context=RequestContext(
                    dict_={'object': self.external_object},
                    request=self.request
                )
            ),
            'no_results_text': _(
                'This view will show the state changes as a workflow '
                'instance is transitioned.'
            ),
            'no_results_title': _(
                'There are no details for this workflow instance'
            ),
            'object': self.external_object.document,
            'title': _('Detail of workflow: %(workflow)s') % {
                'workflow': self.external_object
            },
            'workflow_instance': self.external_object
        }

    def get_external_object_queryset(self):
        document_queryset = AccessControlList.objects.restrict_queryset(
            queryset=Document.valid.all(),
            permission=permission_workflow_template_view,
            user=self.request.user
        )

        return WorkflowInstance.objects.filter(
            document_id__in=document_queryset.values('pk')
        )

    def get_source_queryset(self):
        return self.external_object.log_entries.order_by('-datetime')


class WorkflowInstanceTransitionExecuteView(
    ExternalObjectViewMixin, FormView
):
    external_object_pk_url_kwarg = 'workflow_instance_id'
    external_object_queryset = WorkflowInstance.valid.all()
    form_class = DynamicForm
    template_name = 'appearance/generic_form.html'
    view_icon = icon_workflow_instance_transition

    def form_valid(self, form):
        form_data = form.cleaned_data
        comment = form_data.pop('comment')

        self.external_object.do_transition(
            comment=comment, extra_data=form_data,
            transition=self.get_workflow_template_transition(),
            user=self.request.user
        )
        messages.success(
            message=_(
                'Document "%s" transitioned successfully'
            ) % self.external_object.document, request=self.request
        )
        return HttpResponseRedirect(
            redirect_to=self.get_success_url()
        )

    def get_external_object_queryset_filtered(self):
        queryset = super().get_external_object_queryset_filtered()

        # Filter further down by document access.
        document_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_workflow_instance_transition,
            queryset=Document.valid.all(),
            user=self.request.user
        )

        return queryset.filter(document__in=document_queryset)

    def get_extra_context(self):
        return {
            'navigation_object_list': ('object', 'workflow_instance'),
            'object': self.external_object.document,
            'title': _(
                'Execute transition "%(transition)s" for workflow: '
                '%(workflow)s'
            ) % {
                'transition': self.get_workflow_template_transition(),
                'workflow': self.external_object,
            },
            'workflow_instance': self.external_object
        }

    def get_form_extra_kwargs(self):
        schema = {
            'fields': {
                'comment': {
                    'label': _('Comment'),
                    'class': 'django.forms.CharField', 'kwargs': {
                        'help_text': _(
                            'Optional comment to attach to the transition.'
                        ),
                        'required': False,
                    }
                }
            },
            'widgets': {
                'comment': {
                    'class': 'django.forms.widgets.Textarea',
                    'kwargs': {
                        'attrs': {
                            'rows': 3
                        }
                    }
                }
            }
        }

        for field in self.get_workflow_template_transition().fields.all():
            schema['fields'][field.name] = {
                'class': FIELD_TYPE_MAPPING[field.field_type],
                'help_text': field.help_text,
                'label': field.label,
                'required': field.required
            }
            if field.widget:
                schema['widgets'][field.name] = {
                    'class': WIDGET_CLASS_MAPPING[field.widget],
                    'kwargs': field.get_widget_kwargs()
                }

        return {'schema': schema}

    def get_success_url(self):
        return self.external_object.get_absolute_url()

    def get_workflow_template_transition(self):
        return get_object_or_404(
            klass=self.external_object.get_transition_choices(
                user=self.request.user
            ), pk=self.kwargs['workflow_template_transition_id']
        )


class WorkflowInstanceTransitionSelectView(
    ExternalObjectViewMixin, FormView
):
    external_object_pk_url_kwarg = 'workflow_instance_id'
    external_object_queryset = WorkflowInstance.valid.all()
    form_class = WorkflowInstanceTransitionSelectForm
    template_name = 'appearance/generic_form.html'
    view_icon = icon_workflow_instance_transition_select

    def form_valid(self, form):
        return HttpResponseRedirect(
            redirect_to=reverse(
                viewname='document_states:workflow_instance_transition_execute',
                kwargs={
                    'workflow_instance_id': self.external_object.pk,
                    'workflow_template_transition_id': form.cleaned_data['transition'].pk
                }
            )
        )

    def get_extra_context(self):
        return {
            'navigation_object_list': ('object', 'workflow_instance'),
            'object': self.external_object.document,
            'submit_label': _('Select'),
            'title': _(
                'Select transition for workflow "%(workflow)s" of document "%(document)s"'
            ) % {
                'document': self.external_object.document,
                'workflow': self.external_object
            },
            'workflow_instance': self.external_object
        }

    def get_external_object_queryset_filtered(self):
        queryset = super().get_external_object_queryset_filtered()

        # Filter further down by document access.
        document_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_workflow_instance_transition,
            queryset=Document.valid.all(),
            user=self.request.user
        )

        return queryset.filter(document__in=document_queryset)

    def get_form_extra_kwargs(self):
        return {
            'user': self.request.user,
            'workflow_instance': self.external_object
        }



from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import EventManagerMethodAfter

from .events import event_document_file_downloaded


@method_event(
    event_manager_class=EventManagerMethodAfter,
    event=event_document_file_downloaded,
    target='self'
)
def method_document_get_download_file_object(self):
    # Thin wrapper to make sure the normal views and API views trigger
    # the download event in the same way.
    return self.open(raw=True)



import logging

from django.apps import apps
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from mayan.apps.user_management.permissions import permission_group_view

from .classes import Permission, PermissionNamespace
from .events import event_role_edited

logger = logging.getLogger(name=__name__)


class RoleBusinessLogicMixin:
    def get_group_count(self, user):
        """
        Return the numeric count of groups that have this role contains.
        The count is filtered by access.
        """
        return self.get_groups(user=user).count()

    def get_groups(self, user):
        """
        Return a filtered queryset groups that have this role contains.
        """
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        return AccessControlList.objects.restrict_queryset(
            permission=permission_group_view, queryset=self.groups.all(),
            user=user
        )
    get_group_count.short_description = _('Group count')

    def get_permission_count(self):
        """
        Return the numeric count of permissions that have this role
        has granted. The count is filtered by access.
        """
        return self.permissions.count()
    get_permission_count.short_description = _('Permission count')

    def grant(self, permission):
        self.permissions.add(permission.stored_permission)

    def groups_add(self, queryset, user=None):
        for model_instance in queryset:
            self.groups.add(model_instance)
            event_role_edited.commit(
                action_object=model_instance, actor=user, target=self
            )

    def groups_remove(self, queryset, user=None):
        for model_instance in queryset:
            self.groups.remove(model_instance)
            event_role_edited.commit(
                action_object=model_instance, actor=user, target=self
            )

    def permissions_add(self, queryset, user=None):
        for model_instance in queryset:
            self.permissions.add(model_instance)
            event_role_edited.commit(
                action_object=model_instance, actor=user or self.user,
                target=self
            )

    def permissions_remove(self, queryset, user=None):
        for model_instance in queryset:
            self.permissions.remove(model_instance)
            event_role_edited.commit(
                action_object=model_instance, actor=user or self.user,
                target=self
            )

    def revoke(self, permission):
        self.permissions.remove(permission.stored_permission)


class StoredPermissionBusinessLogicMixin:
    @property
    def label(self):
        try:
            permission = self.volatile_permission
        except KeyError:
            return _('Unknown or obsolete permission: %s') % self.name
        else:
            return permission.label

    @property
    def namespace_label(self):
        try:
            permission_namespace = PermissionNamespace.get(
                name=self.namespace
            )
        except KeyError:
            return _(
                'Unknown or obsolete permission namespace: %s'
            ) % self.namespace
        else:
            return permission_namespace.label

    def user_has_this(self, user):
        """
        Helper method to check if a user has been granted this permission.
        The check is done sequentially over all of the user's groups and
        roles. The check is interrupted at the first positive result.
        The check always returns True for super users or staff users.
        """
        Role = apps.get_model(app_label='permissions', model_name='Role')

        if user.is_superuser or user.is_staff:
            logger.debug(
                'Permission "%s" granted to user "%s" as super user or '
                'staff', self, user
            )
            return True

        if not user.is_authenticated:
            return False

        if Role.objects.filter(groups__user=user, permissions=self).exists():
            return True
        else:
            logger.debug(
                'Fallthru: Permission "%s" not granted to user "%s"',
                self, user
            )
            return False

    @cached_property
    def volatile_permission(self):
        """
        Returns the real class of the permission represented by this model
        instance.
        """
        return Permission.get(pk=self.volatile_permission_id)

    @cached_property
    def volatile_permission_id(self):
        """
        Return the identifier of the real permission class represented by
        this model instance.
        """
        return '{}.{}'.format(self.namespace, self.name)



import logging

from django.template import Variable, VariableDoesNotExist

logger = logging.getLogger(name=__name__)


class TemplateObjectMixin:
    def check_condition(self, context, resolved_object=None):
        """
        Check to see if menu has a conditional display function and return
        the result of the condition function against the context.
        """
        if self.condition:
            return self.condition(
                context=context, resolved_object=resolved_object
            )
        else:
            return True

    def get_request(self, context, request=None):
        if not request:
            # Try to get the request object the faster way and fallback to
            # the slower method.
            try:
                request = context.request
            except AttributeError:
                # Simple request extraction failed. Might not be a view
                # context. Try alternate method.
                try:
                    request = Variable(
                        var='request'
                    ).resolve(context=context)
                except VariableDoesNotExist:
                    # There is no request variable, most probable a 500 in
                    # a test view. Don't return any resolved links then.
                    logger.warning(
                        'No request variable, aborting `{}` '
                        'resolution'.format(self.__class__.__name__)
                    )
                    raise

        return request



from django.contrib.contenttypes.models import ContentType
from django.core.files import File

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.permissions.tests.mixins import PermissionTestMixin
from mayan.apps.testing.tests.mixins import TestMixinObjectCreationTrack

from ..classes import Layer
from ..models import Asset, LayerTransformation
from ..tasks import task_content_object_image_generate
from ..transformations import BaseTransformation

from .literals import (
    TEST_ASSET_LABEL, TEST_ASSET_LABEL_EDITED, TEST_ASSET_INTERNAL_NAME,
    TEST_ASSET_PATH, TEST_LAYER_LABEL, TEST_LAYER_ORDER, TEST_LAYER_NAME,
    TEST_TRANSFORMATION_ARGUMENT, TEST_TRANSFORMATION_ARGUMENT_EDITED,
    TEST_TRANSFORMATION_LABEL, TEST_TRANSFORMATION_NAME
)


class AssetTestMixin(TestMixinObjectCreationTrack):
    _test_object_model = Asset
    _test_object_name = '_test_asset'

    def _create_test_asset(self):
        with open(file=TEST_ASSET_PATH, mode='rb') as file_object:
            self._test_asset = Asset.objects.create(
                label=TEST_ASSET_LABEL,
                internal_name=TEST_ASSET_INTERNAL_NAME,
                file=File(file=file_object)
            )


class AssetAPIViewTestMixin(AssetTestMixin):
    def _request_test_asset_create_api_view(self):
        self._test_object_track()

        with open(file=TEST_ASSET_PATH, mode='rb') as file_object:
            response = self.post(
                viewname='rest_api:asset-list', data={
                    'label': TEST_ASSET_LABEL,
                    'internal_name': TEST_ASSET_INTERNAL_NAME,
                    'file': File(file=file_object)
                }
            )

        self._test_object_set()

        return response

    def _request_test_asset_delete_api_view(self):
        return self.delete(
            viewname='rest_api:asset-detail',
            kwargs={'asset_id': self._test_asset.pk}
        )

    def _request_test_asset_detail_api_view(self):
        return self.get(
            viewname='rest_api:asset-detail',
            kwargs={'asset_id': self._test_asset.pk}
        )

    def _request_test_asset_edit_via_patch_api_view(self):
        return self.patch(
            viewname='rest_api:asset-detail', kwargs={
                'asset_id': self._test_asset.pk
            }, data={'label': TEST_ASSET_LABEL_EDITED}
        )

    def _request_test_asset_edit_via_put_api_view(self):
        with open(file=TEST_ASSET_PATH, mode='rb') as file_object:
            return self.put(
                viewname='rest_api:asset-detail', kwargs={
                    'asset_id': self._test_asset.pk
                }, data={
                    'label': TEST_ASSET_LABEL_EDITED,
                    'internal_name': TEST_ASSET_INTERNAL_NAME,
                    'file': File(file=file_object)
                }
            )

    def _request_test_asset_list_api_view(self):
        return self.get(viewname='rest_api:asset-list')


class AssetTaskTestMixin(AssetTestMixin):
    def _execute_asset_task_content_object_image_generate(self):
        content_type = ContentType.objects.get_for_model(model=Asset)

        task_content_object_image_generate.apply_async(
            kwargs={
                'content_type_id': content_type.pk,
                'object_id': self._test_asset.pk
            }
        ).get()


class AssetViewTestMixin(AssetTestMixin):
    def _request_test_asset_create_view(self):
        self._test_object_track()

        with open(file=TEST_ASSET_PATH, mode='rb') as file_object:
            response = self.post(
                viewname='converter:asset_create', data={
                    'label': TEST_ASSET_LABEL,
                    'internal_name': TEST_ASSET_INTERNAL_NAME,
                    'file': file_object
                }
            )

        self._test_object_set()

        return response

    def _request_test_asset_delete_view(self):
        return self.post(
            viewname='converter:asset_single_delete', kwargs={
                'asset_id': self._test_asset.pk
            }
        )

    def _request_test_asset_detail_view(self):
        return self.get(
            viewname='converter:asset_detail', kwargs={
                'asset_id': self._test_asset.pk
            }
        )

    def _request_test_asset_edit_view(self):
        return self.post(
            viewname='converter:asset_edit', kwargs={
                'asset_id': self._test_asset.pk
            }, data={
                'label': TEST_ASSET_LABEL_EDITED,
                'internal_name': TEST_ASSET_INTERNAL_NAME,
            }
        )

    def _request_test_asset_list_view(self):
        return self.get(viewname='converter:asset_list')


class LayerTestCaseMixin:
    def setUp(self):
        super().setUp()
        Layer.invalidate_cache()


class LayerTestMixin(PermissionTestMixin):
    _test_transformation_object = None
    _test_transformation_object_parent = None
    auto_create_test_layer = True

    def setUp(self):
        super().setUp()

        self._test_transformation_object = self._test_document_version_page
        self._test_transformation_object_parent = self._test_document

        self._test_transformation_object_content_type = ContentType.objects.get_for_model(
            model=self._test_transformation_object
        )

        if self.auto_create_test_layer:
            self._create_test_permission()
            self._test_layer_permission_create = self._test_permission

            self._create_test_permission()
            self._test_layer_permission_delete = self._test_permission

            self._create_test_permission()
            self._test_layer_permission_edit = self._test_permission

            self._create_test_permission()
            self._test_layer_permission_view = self._test_permission

            self._test_layer = Layer(
                label=TEST_LAYER_LABEL, name=TEST_LAYER_NAME,
                order=TEST_LAYER_ORDER, permissions={
                    'create': self._test_layer_permission_create,
                    'delete': self._test_layer_permission_delete,
                    'edit': self._test_layer_permission_edit,
                    'select': self._test_layer_permission_create,
                    'view': self._test_layer_permission_view
                }
            )

            ModelPermission.register(
                model=self._test_transformation_object_parent._meta.model,
                permissions=(
                    self._test_layer_permission_create,
                    self._test_layer_permission_delete,
                    self._test_layer_permission_edit,
                    self._test_layer_permission_view
                )
            )

    def tearDown(self):
        if self.auto_create_test_layer:
            Layer._registry.pop(self._test_layer.name, None)

        super().tearDown()


class TransformationTestMixin(LayerTestMixin, TestMixinObjectCreationTrack):
    _test_object_model = LayerTransformation
    _test_object_name = '_test_transformation'
    auto_create_test_transformation_class = True

    def setUp(self):
        super().setUp()
        if self.auto_create_test_transformation_class:
            self._create_test_transformation_class()

        BaseTransformation.register(
            layer=self._test_layer,
            transformation=self.TestTransformationClass
        )

    def _create_test_transformation(self):
        self._test_transformation = self._test_layer.add_transformation_to(
            obj=self._test_transformation_object,
            transformation_class=self.TestTransformationClass,
            arguments=getattr(
                self, '_test_transformation_arguments',
                TEST_TRANSFORMATION_ARGUMENT
            )
        )

    def _create_test_transformation_class(self):
        class TestTransformation(BaseTransformation):
            arguments = ('test_argument',)
            label = TEST_TRANSFORMATION_LABEL
            name = TEST_TRANSFORMATION_NAME

            def execute_on(self, *args, **kwargs):
                super().execute_on(*args, **kwargs)

                return self.image

        self.TestTransformationClass = TestTransformation


class TransformationViewTestMixin(TransformationTestMixin):
    def _request_transformation_create_post_view(self):
        self._test_object_track()

        response = self.post(
            viewname='converter:transformation_create', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name,
                'transformation_name': self.TestTransformationClass.name
            }, data={
                'arguments': getattr(
                    self, '.test_transformation_argument',
                    TEST_TRANSFORMATION_ARGUMENT
                )
            }
        )

        self._test_object_set()

        return response

    def _request_transformation_create_get_view(self):
        return self.get(
            viewname='converter:transformation_create', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name,
                'transformation_name': self.TestTransformationClass.name
            }, data={
                'arguments': getattr(
                    self, '.test_transformation_argument',
                    TEST_TRANSFORMATION_ARGUMENT
                )
            }
        )

    def _request_transformation_delete_view(self):
        return self.post(
            viewname='converter:transformation_delete', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name,
                'transformation_id': self._test_transformation.pk
            }
        )

    def _request_transformation_edit_view(self):
        return self.post(
            viewname='converter:transformation_edit', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name,
                'transformation_id': self._test_transformation.pk
            }, data={
                'arguments': getattr(
                    self, '_test_transformation_argument_edited',
                    TEST_TRANSFORMATION_ARGUMENT_EDITED
                )
            }
        )

    def _request_transformation_list_view(self):
        return self.get(
            viewname='converter:transformation_list', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name
            }
        )

    def _request_test_transformation_select_get_view(self):
        return self.get(
            viewname='converter:transformation_select', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name
            }
        )

    def _request_test_transformation_select_post_view(self):
        return self.post(
            viewname='converter:transformation_select', kwargs={
                'app_label': self._test_transformation_object_content_type.app_label,
                'model_name': self._test_transformation_object_content_type.model,
                'object_id': self._test_transformation_object.pk,
                'layer_name': self._test_layer.name
            }, data={
                'transformation': self.TestTransformationClass.name
            }
        )



TEST_ADMIN_USER_EMAIL = 'testemail@example.com'
TEST_ADMIN_USER_PASSWORD = 'test admin user password'
TEST_ADMIN_USER_USERNAME = 'test_admin_user_username'
TEST_FIRST_TIME_LOGIN_TEXT = 'Automatic credentials'
TEST_MOCK_VIEW_TEXT = 'mock view text'



from django.utils.translation import ugettext_lazy as _

from ..icons import icon_document_recently_accessed_list
from ..models.recently_accessed_document_models import RecentlyAccessedDocument

from .document_views import DocumentListView


class RecentlyAccessedDocumentListView(DocumentListView):
    view_icon = icon_document_recently_accessed_list

    def get_document_queryset(self):
        return RecentlyAccessedDocument.valid.get_for_user(
            user=self.request.user
        )

    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'no_results_icon': icon_document_recently_accessed_list,
                'no_results_text': _(
                    'This view will list the latest documents viewed or '
                    'manipulated in any way by this user account.'
                ),
                'no_results_title': _(
                    'There are no recently accessed documents'
                ),
                'title': _('Recently accessed')
            }
        )
        return context



from django.contrib import admin

from .models import DownloadFile, SharedUploadedFile


@admin.register(DownloadFile)
class DownloadFileAdmin(admin.ModelAdmin):
    date_hierarchy = 'datetime'
    list_display = ('file', 'filename', 'datetime',)
    readonly_fields = list_display


@admin.register(SharedUploadedFile)
class SharedUploadedFileAdmin(admin.ModelAdmin):
    date_hierarchy = 'datetime'
    list_display = ('file', 'filename', 'datetime',)
    readonly_fields = list_display



from django import forms
from django.forms.formsets import formset_factory
from django.utils.translation import ugettext_lazy as _


class DocumentDownloadForm(forms.Form):
    document_file_id = forms.CharField(
        label=_('Document file ID'), widget=forms.HiddenInput
    )
    document = forms.CharField(
        label=_('Document'), required=False,
        widget=forms.TextInput(
            attrs={
                'readonly': 'readonly'
            }
        )
    )
    document_file = forms.CharField(
        label=_('Document file'), required=False,
        widget=forms.TextInput(
            attrs={
                'readonly': 'readonly'
            }
        )
    )
    include = forms.BooleanField(
        initial=True, label=_('Include'), required=False
    )


DocumentDownloadFormSet = formset_factory(form=DocumentDownloadForm, extra=0)



from mayan.apps.appearance.classes import Icon

icon_comment_add = Icon(
    driver_name='fontawesome-dual', primary_symbol='comment',
    secondary_symbol='plus'
)
icon_comment_delete = Icon(driver_name='fontawesome', symbol='times')
icon_comment_detail = Icon(driver_name='fontawesome', symbol='comment')
icon_comment_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_comments_for_document = Icon(
    driver_name='fontawesome', symbol='comment'
)



from django.core import management
from django.utils.translation import ugettext_lazy as _

from ...utils import PassthroughStorageProcessor


class Command(management.BaseCommand):
    help = 'Process model files over a storage pipeline.'

    def add_arguments(self, parser):
        parser.add_argument(
            '--app', action='store', dest='app_label',
            help=_('Name of the app to process.'),
            required=True,
        )
        parser.add_argument(
            '--log', action='store', dest='log_file',
            help=_(
                'Path of the database (.dbm) file that will be created/read '
                'to keep track of items processed.'
            ),
            required=True,
        )
        parser.add_argument(
            '--model', action='store', dest='model_name',
            help=_('Process a specific model.'),
            required=True,
        )
        parser.add_argument(
            '--reverse', action='store_true', dest='reverse',
            help=_(
                'Process the files in reverse to undo the storage '
                'pipeline transformations.'
            )
        )
        parser.add_argument(
            '--storage_name', action='store', dest='defined_storage_name',
            help=_('Name of the storage to process.'),
            required=True,
        )

    def handle(self, *args, **options):
        processor = PassthroughStorageProcessor(
            app_label=options['app_label'],
            defined_storage_name=options['defined_storage_name'],
            log_file=options['log_file'], model_name=options['model_name'],
        )
        processor.execute(reverse=options['reverse'])



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Workflows'), name='document_states'
)

event_workflow_instance_created = namespace.add_event_type(
    label=_('Workflow instance created'), name='workflow_instance_created'
)
event_workflow_instance_transitioned = namespace.add_event_type(
    label=_('Workflow instance transitioned'),
    name='workflow_instance_transitioned'
)

event_workflow_template_created = namespace.add_event_type(
    label=_('Workflow created'), name='workflow_created'
)
event_workflow_template_edited = namespace.add_event_type(
    label=_('Workflow edited'), name='workflow_edited'
)



from django.db import migrations


def code_create_file_metadata_setting_for_existing_document_types(apps, schema_editor):
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )
    DocumentTypeSettings = apps.get_model(
        app_label='file_metadata', model_name='DocumentTypeSettings'
    )

    for document_type in DocumentType.objects.using(alias=schema_editor.connection.alias).all():
        try:
            DocumentTypeSettings.objects.using(
                alias=schema_editor.connection.alias
            ).get_or_create(document_type=document_type)
        except DocumentTypeSettings.DoesNotExist:
            pass


def code_delete_file_metadata_setting_for_existing_document_types(apps, schema_editor):
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )
    DocumentTypeSettings = apps.get_model(
        app_label='file_metadata', model_name='DocumentTypeSettings'
    )

    for document_type in DocumentType.objects.using(alias=schema_editor.connection.alias).all():
        try:
            DocumentTypeSettings.objects.using(
                alias=schema_editor.connection.alias
            ).get(document_type=document_type).delete()
        except DocumentTypeSettings.DoesNotExist:
            pass


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0047_auto_20180917_0737'),
        ('file_metadata', '0001_initial')
    ]

    operations = [
        migrations.RunPython(
            code=code_create_file_metadata_setting_for_existing_document_types,
            reverse_code=code_delete_file_metadata_setting_for_existing_document_types
        )
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_a, worker_c

queue_events_fast = CeleryQueue(
    label=_('Events fast'), name='events_fast', worker=worker_a
)
queue_events_slow = CeleryQueue(
    label=_('Events slow'), name='events_slow', transient=True,
    worker=worker_c
)

queue_events_fast.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_commit',
    label=_('Commit an event'), name='task_event_commit'
)

queue_events_slow.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_queryset_clear',
    label=_('Clear event querysets'), name='task_event_queryset_clear'
)
queue_events_slow.add_task_type(
    dotted_path='mayan.apps.events.tasks.task_event_queryset_export',
    label=_('Export event querysets'), name='task_event_queryset_export'
)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0002_auto_20150608_1902'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='autoadminsingleton',
            name='account',
        ),
        migrations.DeleteModel(
            name='AutoAdminSingleton',
        ),
    ]



from mayan.apps.documents.api_views.api_view_mixins import ParentObjectDocumentFilePageAPIViewMixin
from mayan.apps.rest_api import generics

from .models import DocumentFilePageContent, DocumentTypeSettings
from .permissions import (
    permission_document_file_content_view,
    permission_document_type_parsing_setup
)
from .serializers import (
    DocumentFilePageContentSerializer, DocumentTypeParsingSettingsSerializer
)


class APIDocumentFilePageContentView(
    ParentObjectDocumentFilePageAPIViewMixin, generics.RetrieveAPIView
):
    """
    get: Returns the content of the selected document page.
    """
    mayan_object_permissions = {
        'GET': (permission_document_file_content_view,),
    }
    serializer_class = DocumentFilePageContentSerializer

    def get_object(self):
        document_file_page = self.get_document_file_page(
            permission=self.mayan_object_permissions[self.request.method][0]
        )

        try:
            return DocumentFilePageContent.objects.get(
                document_file_page=document_file_page
            )
        except DocumentFilePageContent.DoesNotExist:
            return DocumentFilePageContent(
                document_file_page=document_file_page
            )


class APIDocumentTypeParsingSettingsView(generics.RetrieveUpdateAPIView):
    """
    get: Return the document type parsing settings.
    patch: Set the document type parsing settings.
    put: Set the document type parsing settings.
    """
    lookup_field = 'document_type__pk'
    lookup_url_kwarg = 'document_type_id'
    mayan_object_permissions = {
        'GET': (permission_document_type_parsing_setup,),
        'PATCH': (permission_document_type_parsing_setup,),
        'PUT': (permission_document_type_parsing_setup,)
    }
    serializer_class = DocumentTypeParsingSettingsSerializer
    source_queryset = DocumentTypeSettings.objects.all()



TEST_DOCUMENT_FILE_METADATA_KEY = 'test_file_metadata_key'
TEST_DOCUMENT_FILE_METADATA_VALUE = 'test file metadata value'

TEST_DRIVER_INTERNAL_NAME = 'exiftool'
TEST_FILE_METADATA_KEY = 'FileType'
TEST_FILE_METADATA_VALUE = 'PNG'
TEST_FILE_METADATA_INDEX_NODE_TEMPLATE = "{{{{ document.file_metadata_value_of.{}__{} }}}}".format(
    TEST_DRIVER_INTERNAL_NAME, TEST_FILE_METADATA_KEY
)

# MSG

TEST_MSG_FILE_METADATA_DOTTED_NAME_SUBJECT = 'extract_msg__subject'
TEST_MSG_FILE_METADATA_DOTTED_NAME_TO = 'extract_msg__to'

TEST_MSG_FILE_METADATA_VALUE_SUBJECT = 'MSG Test File'
TEST_MSG_FILE_METADATA_VALUE_TO = 'time2talk@online-convert.com <time2talk@online-convert.com>'

# PDF

TEST_PDF_FILE_METADATA_DOTTED_NAME = 'exiftool__Producer'
TEST_PDF_FILE_METADATA_VALUE = 'pdfTeX-1.40.10'

TEST_STORED_DRIVER_PATH = 'mayan.apps.file_metadata.tests.drivers'
TEST_STORED_DRIVER_INTERNAL_NAME = 'test_stored_driver'



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models.index_template_models import IndexTemplate, IndexTemplateNode


class IndexTemplateNodeInline(admin.StackedInline):
    extra = 0
    list_display = ('expression', 'enabled', 'link_documents')
    model = IndexTemplateNode


@admin.register(IndexTemplate)
class IndexTemplateAdmin(admin.ModelAdmin):
    filter_horizontal = ('document_types',)
    inlines = (IndexTemplateNodeInline,)
    list_display = ('label', 'enabled', 'get_document_types')

    def get_document_types(self, instance):
        return ', '.join(
            [
                '"{}"'.format(document_type) for document_type in instance.document_types.all()
            ]
        ) or _('None')

    get_document_types.short_description = _('Document types')



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            'document_indexing',
            '0028_populate_existing_index_template_event_triggers'
        )
    ]

    operations = [
        migrations.AlterField(
            model_name='indexinstancenode', name='value',
            field=models.CharField(
                blank=True, db_index=True, max_length=255,
                verbose_name='Value'
            )
        )
    ]



from django.db.models import Q

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin

from ...models import DocumentMetadata

from ..literals import TEST_METADATA_VALUE, TEST_METADATA_VALUE_EDITED

from .metadata_type_mixins import MetadataTypeTestMixin


class DocumentMetadataMixin(DocumentTestMixin, MetadataTypeTestMixin):
    _test_document_metadata_value = TEST_METADATA_VALUE

    def _create_test_document_metadata(self):
        self._test_document_metadata = self._test_document.metadata.create(
            metadata_type=self._test_metadata_type,
            value=self._test_document_metadata_value
        )


class DocumentMetadataAPIViewTestMixin(DocumentMetadataMixin):
    def _request_document_metadata_create_api_view(self, extra_data=None):
        pk_list = list(
            DocumentMetadata.objects.values_list('pk', flat=True)
        )

        data = {
            'metadata_type_id': self._test_metadata_type.pk,
            'value': TEST_METADATA_VALUE
        }
        data.update(
            extra_data or {}
        )

        response = self.post(
            viewname='rest_api:documentmetadata-list',
            kwargs={'document_id': self._test_document.pk}, data=data
        )

        try:
            self._test_document_metadata = DocumentMetadata.objects.get(
                ~Q(pk__in=pk_list)
            )
        except DocumentMetadata.DoesNotExist:
            self._test_document_metadata = None

        return response

    def _request_document_metadata_delete_api_view(self):
        return self.delete(
            viewname='rest_api:documentmetadata-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'metadata_id': self._test_document_metadata.pk
            }
        )

    def _request_document_metadata_edit_api_view_via_patch(
        self, extra_data=None
    ):
        data = {
            'value': TEST_METADATA_VALUE_EDITED
        }
        data.update(
            extra_data or {}
        )

        return self.patch(
            viewname='rest_api:documentmetadata-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'metadata_id': self._test_document_metadata.pk
            }, data=data
        )

    def _request_document_metadata_edit_api_view_via_put(
        self, extra_data=None
    ):
        data = {
            'value': TEST_METADATA_VALUE_EDITED
        }
        data.update(
            extra_data or {}
        )

        return self.put(
            viewname='rest_api:documentmetadata-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'metadata_id': self._test_document_metadata.pk
            }, data=data
        )

    def _request_document_metadata_list_api_view(self):
        return self.get(
            viewname='rest_api:documentmetadata-list', kwargs={
                'document_id': self._test_document.pk
            }
        )


class DocumentMetadataViewTestMixin(DocumentMetadataMixin):
    def _request_test_document_metadata_add_get_view(self):
        return self.get(
            viewname='metadata:metadata_add', kwargs={
                'document_id': self._test_document.pk
            }, data={'metadata_type': self._test_metadata_type.pk}
        )

    def _request_test_document_metadata_add_post_view(self):
        return self.post(
            viewname='metadata:metadata_add', kwargs={
                'document_id': self._test_document.pk
            }, data={'metadata_type': self._test_metadata_type.pk}
        )

    def _request_test_document_metadata_multiple_add_post_view(self):
        return self.post(
            viewname='metadata:metadata_add', kwargs={
                'document_id': self._test_document.pk
            }, data={
                'metadata_type': [
                    metadata_type.pk for metadata_type in self._test_metadata_type_list
                ]
            }
        )

    def _request_test_document_metadata_edit_post_view(
        self, extra_data=None, follow=False
    ):
        data = {
            'form-0-metadata_type_id': self._test_metadata_type.pk,
            'form-0-update': True,
            'form-0-value': TEST_METADATA_VALUE_EDITED,
            'form-TOTAL_FORMS': '1',
            'form-INITIAL_FORMS': '0',
            'form-MAX_NUM_FORMS': ''
        }

        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='metadata:metadata_edit', kwargs={
                'document_id': self._test_document.pk
            }, data=data, follow=follow
        )

    def _request_test_document_metadata_list_view(self):
        return self.get(
            viewname='metadata:metadata_list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_document_metadata_remove_get_view(self):
        return self.get(
            viewname='metadata:metadata_remove', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_document_metadata_remove_post_view(self, index=0):
        return self.post(
            viewname='metadata:metadata_remove',
            kwargs={'document_id': self._test_document.pk}, data={
                'form-0-metadata_type_id': self._test_metadata_type_list[index].pk,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': ''
            }
        )

    def _request_test_document_multiple_metadata_add_post_view(self):
        return self.post(
            viewname='metadata:metadata_multiple_add', data={
                'id_list': '{},{}'.format(
                    self._test_documents[0].pk, self._test_documents[1].pk
                ), 'metadata_type': self._test_metadata_type.pk
            }
        )

    def _request_test_document_multiple_metadata_edit_get_view(self):
        return self.get(
            viewname='metadata:metadata_multiple_edit', data={
                'id_list': '{},{}'.format(
                    self._test_documents[0].pk, self._test_documents[1].pk
                )
            }
        )

    def _request_test_document_multiple_metadata_edit_post_view(self):
        return self.post(
            viewname='metadata:metadata_multiple_edit', data={
                'id_list': '{},{}'.format(
                    self._test_documents[0].pk, self._test_documents[1].pk
                ),
                'form-0-metadata_type_id': self._test_metadata_type.pk,
                'form-0-update': True,
                'form-0-value': TEST_METADATA_VALUE_EDITED,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': ''
            }
        )

    def _request_test_document_multiple_metadata_remove_get_view(self):
        return self.get(
            viewname='metadata:metadata_multiple_remove', data={
                'id_list': '{},{}'.format(
                    self._test_documents[0].pk, self._test_documents[0].pk
                )
            }
        )

    def _request_test_document_multiple_metadata_remove_post_view(self):
        return self.post(
            viewname='metadata:metadata_multiple_remove', data={
                'id_list': '{},{}'.format(
                    self._test_documents[0].pk, self._test_documents[1].pk
                ),
                'form-0-metadata_type_id': self._test_metadata_type.pk,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': ''
            }
        )



class DashboardViewTestMixin:
    def _request_test_dashboard_detail_view(self):
        return self.get(
            viewname='dashboards:dashboard_detail', kwargs={
                'dashboard_name': self.test_dashboard.name
            }
        )

    def _request_test_dashboard_list_view(self):
        return self.get(viewname='dashboards:dashboard_list')



#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import mayan

PACKAGE_NAME = 'mayan-edms'
PACKAGE_DIR = 'mayan'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


def find_packages(directory):
    # Compile the list of packages available, because distutils doesn't have
    # an easy way to do this.
    packages, data_files = [], []
    root_dir = os.path.dirname(__file__)
    if root_dir != '':
        os.chdir(root_dir)

    for dirpath, dirnames, filenames in os.walk(directory):
        if not dirpath.startswith('mayan/media'):
            # Ignore dirnames that start with '.'
            if os.path.basename(dirpath).startswith('.'):
                continue
            if '__init__.py' in filenames:
                packages.append('.'.join(fullsplit(dirpath)))
            elif filenames:
                data_files.append(
                    [
                        dirpath, [
                            os.path.join(dirpath, filename) for filename in filenames
                        ]
                    ]
                )

    return packages


install_requires = """
django==3.2.22
CairoSVG==2.5.2
Pillow==10.0.0
PyYAML==6.0.1
Whoosh==2.7.4
bleach==6.0.0
boto3==1.28.16
celery==5.3.1
dateparser==1.1.8
django-activity-stream==1.4.0
django-auth-ldap==4.4.0
django-celery-beat==2.3.0
django-cors-headers==4.2.0
django-formtools==2.3
django-mathfilters==1.0.0
django-model-utils==4.3.1
django-mptt==0.14.0
django-qsstats-magic==1.1.0
django-solo==2.0.0
django-storages==1.13.2
django-stronghold==0.4.0
django-widget-tweaks==1.4.12
djangorestframework==3.14.0
djangorestframework-recursive==0.1.2
drf-yasg==1.21.7
elasticsearch==7.17.9
elasticsearch-dsl==7.4.1
extract-msg==0.37.1
flanker==0.9.11
flex==6.14.1
furl==2.1.3
fusepy==3.0.1
gevent==22.10.2
graphviz==0.17
greenlet==2.0.2
gunicorn==21.2.0
importlib-metadata==6.8.0
jsonschema==4.18.0
mozilla-django-oidc==2.0.0
node-semver==0.8.1
pycountry==22.3.5
pycryptodome==3.18.0
pyotp==2.6.0
pypdf==3.14.0
python-dateutil==2.8.2
python-magic==0.4.27
python_gnupg==0.4.8
pytz==2022.1
qrcode==7.3.1
requests==2.31.0
sentry-sdk==1.29.0
sh==2.0.4
swagger-spec-validator==3.0.3
whitenoise==6.5.0
""".split()

with open(file='README.rst') as file_object:
    readme = file_object.read()

with open(file='HISTORY.rst') as file_object:
    history = file_object.read()

setup(
    author='Roberto Rosario',
    author_email='roberto.rosario@mayan-edms.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Education',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Communications :: File Sharing',
    ],
    description=mayan.__description__,
    include_package_data=True,
    install_requires=install_requires,
    license='Apache 2.0',
    long_description=readme + '\n\n' + history,
    name=PACKAGE_NAME,
    packages=find_packages(PACKAGE_DIR),
    platforms=['any'],
    project_urls={
        'Documentation': 'https://docs.mayan-edms.com/',
        'Changelog': 'https://gitlab.com/mayan-edms/mayan-edms/-/blob/master/HISTORY.rst',
        'Bug Tracker': 'https://gitlab.com/mayan-edms/mayan-edms/-/issues',
        'Source Code': 'https://gitlab.com/mayan-edms/mayan-edms',
        'Support': 'https://www.mayan-edms.com/support/'
    },
    python_requires='>=3.9',
    scripts=['mayan/bin/mayan-edms.py'],
    url=mayan.__website__,
    version=mayan.__version__,
    zip_safe=False,
)



from django.apps import apps


def handler_user_locale_profile_create(sender, instance, created, **kwargs):
    UserLocaleProfile = apps.get_model(
        app_label='locales', model_name='UserLocaleProfile'
    )

    if created:
        UserLocaleProfile.objects.create(user=instance)



from mayan.apps.documents.tests.base import BaseTestCase

from .mixins import AssetTaskTestMixin


class AssetTasksTestCase(AssetTaskTestMixin, BaseTestCase):
    def setUp(self):
        super().setUp()
        self._create_test_asset()

    def test_asset_task_content_object_image_generate(self):
        self._clear_events()

        self._execute_asset_task_content_object_image_generate()

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.messaging.events import event_message_created
from mayan.apps.messaging.models import Message
from mayan.apps.storage.events import event_download_file_created
from mayan.apps.storage.models import DownloadFile

from ..events import event_document_file_downloaded
from ..permissions import permission_document_file_download

from .mixins import (
    DocumentDownloadViewTestMixin, DocumentFileDownloadViewTestMixin
)


class DocumentDownloadViewTestCase(
    DocumentDownloadViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_download_get_view_no_permission(self):
        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_get_view()
        self.assertNotContains(
            response=response, status_code=200, text=str(
                self._test_document_file
            )
        )

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_download_get_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_download
        )

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_get_view()
        self.assertContains(
            response=response, status_code=200, text=str(
                self._test_document_file
            )
        )
        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_download_get_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        self._test_document.delete()

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_get_view()
        self.assertNotContains(
            response=response, status_code=200, text=str(
                self._test_document_file
            )
        )

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_download_post_view_no_permission(self):
        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_download_post_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_download_file = DownloadFile.objects.first()
        test_message = Message.objects.first()

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_download_file)
        self.assertEqual(events[0].verb, event_download_file_created.id)

        self.assertEqual(events[1].action_object, test_download_file)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_file)
        self.assertEqual(events[1].verb, event_document_file_downloaded.id)

        self.assertEqual(events[2].action_object, None)
        self.assertEqual(events[2].actor, test_message)
        self.assertEqual(events[2].target, test_message)
        self.assertEqual(events[2].verb, event_message_created.id)

    def test_trashed_document_download_post_view_no_permission(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        self._test_document.delete()

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_download_post_view_with_single_file_access(self):
        self.grant_access(
            obj=self._test_document_file_list[0],
            permission=permission_document_file_download
        )

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_download_file = DownloadFile.objects.first()
        test_message = Message.objects.first()

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_download_file)
        self.assertEqual(events[0].verb, event_download_file_created.id)

        self.assertEqual(events[1].action_object, test_download_file)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_file)
        self.assertEqual(events[1].verb, event_document_file_downloaded.id)

        self.assertEqual(events[2].action_object, None)
        self.assertEqual(events[2].actor, test_message)
        self.assertEqual(events[2].target, test_message)
        self.assertEqual(events[2].verb, event_message_created.id)


class DocumentMultipleDownloadViewTestCase(
    DocumentDownloadViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_multiple_download_get_view_no_permission(self):
        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_get_view()
        self.assertNotContains(
            response=response, status_code=200, text=str(
                self._test_document_file
            )
        )

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_multiple_download_get_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_get_view()
        self.assertContains(
            response=response, status_code=200, text=str(
                self._test_document_file
            )
        )

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_multiple_download_get_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        self._test_document.delete()

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_get_view()
        self.assertNotContains(
            response=response, status_code=200, text=str(
                self._test_document_file
            )
        )

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_multiple_download_post_view_no_permission(self):
        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_multiple_download_post_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_download_file = DownloadFile.objects.first()
        test_message = Message.objects.first()

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_download_file)
        self.assertEqual(events[0].verb, event_download_file_created.id)

        self.assertEqual(events[1].action_object, test_download_file)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_file)
        self.assertEqual(events[1].verb, event_document_file_downloaded.id)

        self.assertEqual(events[2].action_object, None)
        self.assertEqual(events[2].actor, test_message)
        self.assertEqual(events[2].target, test_message)
        self.assertEqual(events[2].verb, event_message_created.id)

    def test_trashed_document_multiple_download_post_view_no_permission(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        self._test_document.delete()

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_multiple_download_post_view_with_single_file_access(self):
        self.grant_access(
            obj=self._test_document_file_list[0],
            permission=permission_document_file_download
        )

        download_file_count = DownloadFile.objects.count()

        self._clear_events()

        response = self._request_test_document_multiple_download_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            DownloadFile.objects.count(), download_file_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_download_file = DownloadFile.objects.first()
        test_message = Message.objects.first()

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, test_download_file)
        self.assertEqual(events[0].verb, event_download_file_created.id)

        self.assertEqual(events[1].action_object, test_download_file)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_file)
        self.assertEqual(events[1].verb, event_document_file_downloaded.id)

        self.assertEqual(events[2].action_object, None)
        self.assertEqual(events[2].actor, test_message)
        self.assertEqual(events[2].target, test_message)
        self.assertEqual(events[2].verb, event_message_created.id)


class DocumentFileDownloadViewTestCase(
    DocumentFileDownloadViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_file_download_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_download_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_download_view_with_permission(self):
        # Set the expected_content_types for
        # common.tests.mixins.ContentTypeCheckMixin
        self.expected_content_types = (
            self._test_document.file_latest.mimetype,
        )

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        self._clear_events()

        response = self._request_test_document_file_download_view()
        self.assertEqual(response.status_code, 200)

        with self._test_document.file_latest.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                filename=self._test_document.file_latest.filename,
                mime_type=self._test_document.file_latest.mimetype
            )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(events[0].verb, event_document_file_downloaded.id)

    def test_trashed_document_file_download_view_with_permission(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_download
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_download_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from ..models.document_models import RecentlyCreatedDocument

from .base import GenericDocumentTestCase


class RecentlyCreatedDocumentModelTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    def test_trashed_document_created_document_add(self):
        self._create_test_document_stub()
        self._test_document.delete()

        self.assertFalse(
            RecentlyCreatedDocument.valid.filter(
                pk=self._test_document.pk
            ).exists()
        )



from datetime import timedelta

from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_b, worker_c

from .literals import (
    CHECK_DELETE_PERIOD_INTERVAL, CHECK_TRASH_PERIOD_INTERVAL,
    INTERVAL_TASK_STUBS_DELETION
)

queue_documents_fast = CeleryQueue(
    name='documents_fast', label=_('Documents fast'), worker=worker_b
)
queue_documents_file = CeleryQueue(
    name='documents_file', label=_('Documents file'), worker=worker_b
)
queue_documents_file_slow = CeleryQueue(
    name='documents_file_slow', label=_('Documents file slow'),
    worker=worker_b
)
queue_documents_periodic = CeleryQueue(
    name='documents_periodic', label=_('Documents periodic'), transient=True,
    worker=worker_c
)
queue_documents_trash = CeleryQueue(
    name='documents_trash', label=_('Documents trash'), worker=worker_c
)
queue_documents_version = CeleryQueue(
    name='documents_version', label=_('Documents version'), worker=worker_b
)

queue_documents_fast.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_create',
    label=_('Create new document file')
)
queue_documents_fast.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_upload',
    label=_('Upload new document file')
)
queue_documents_fast.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_tasks.task_document_upload',
    label=_('Upload new document')
)

queue_documents_file.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_checksum_update',
    label=_('Calculate and update document file checksum')
)
queue_documents_file.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_mimetype_update',
    label=_('Calculate and update document file MIME type')
)
queue_documents_file.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_page_count_update',
    label=_('Update document file page count')
)
queue_documents_file.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_size_update',
    label=_('Update document file size')
)
queue_documents_file.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_version_create',
    label=_('Create a new version after the file is processed')
)

queue_documents_file_slow.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_file_tasks.task_document_file_delete',
    label=_('Delete a document file')
)

queue_documents_periodic.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_type_tasks.task_document_type_document_trash_periods_check',
    label=_('Check document type trash periods'),
    name='task_document_type_document_trash_periods_check',
    schedule=timedelta(seconds=CHECK_TRASH_PERIOD_INTERVAL),
)
queue_documents_periodic.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_type_tasks.task_document_type_document_stubs_delete',
    label=_('Delete document stubs'),
    name='task_document_type_document_stubs_delete',
    schedule=timedelta(seconds=INTERVAL_TASK_STUBS_DELETION),
)
queue_documents_periodic.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_type_tasks.task_document_type_trashed_document_delete_periods_check',
    label=_('Check document type delete periods'),
    name='task_document_type_trashed_document_delete_periods_check',
    schedule=timedelta(
        seconds=CHECK_DELETE_PERIOD_INTERVAL
    )
)

queue_documents_trash.add_task_type(
    dotted_path='mayan.apps.documents.tasks.trashed_document_tasks.task_trash_can_empty',
    label=_('Empty the trash can')
)
queue_documents_trash.add_task_type(
    dotted_path='mayan.apps.documents.tasks.trashed_document_tasks.task_trashed_document_delete',
    label=_('Delete a document')
)

queue_documents_version.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_version_tasks.task_document_version_page_list_append',
    label=_('Append all document file pages to a document version')
)
queue_documents_version.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_version_tasks.task_document_version_page_list_reset',
    label=_('Reset the page list of a document version')
)
queue_documents_version.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_version_tasks.task_document_version_export',
    label=_('Export a document version')
)
queue_documents_version.add_task_type(
    dotted_path='mayan.apps.documents.tasks.document_version_tasks.task_document_version_delete',
    label=_('Delete a document version')
)



from .base import GenericDocumentTestCase

from ..events import event_document_version_page_deleted
from ..models.document_version_page_models import DocumentVersionPage


class DocumentFilePageTestCase(GenericDocumentTestCase):
    def test_method_delete(self):
        document_version_page_count = DocumentVersionPage.objects.count()

        self._clear_events()

        self._test_document_file_page._event_actor = self._test_case_user
        self._test_document_file_page.delete()

        self.assertEqual(
            DocumentVersionPage.objects.count(),
            document_version_page_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )



from django.db.models import Q

from ..models import Announcement

from .literals import (
    TEST_ANNOUNCEMENT_LABEL, TEST_ANNOUNCEMENT_LABEL_EDITED,
    TEST_ANNOUNCEMENT_TEXT, TEST_ANNOUNCEMENT_TEXT_EDITED
)


class AnnouncementAPIViewTestMixin:
    def _request_announcement_create_view(self):
        pk_list = list(
            Announcement.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='rest_api:announcement-list', data={
                'label': TEST_ANNOUNCEMENT_LABEL,
                'text': TEST_ANNOUNCEMENT_TEXT
            }
        )

        try:
            self._test_announcement = Announcement.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Announcement.DoesNotExist:
            self._test_announcement = None

        return response

    def _request_announcement_delete_view(self):
        return self.delete(
            viewname='rest_api:announcement-detail', kwargs={
                'announcement_id': self._test_announcement.pk
            }
        )

    def _request_announcement_detail_view(self):
        return self.get(
            viewname='rest_api:announcement-detail', kwargs={
                'announcement_id': self._test_announcement.pk
            }
        )

    def _request_announcement_edit_via_patch_view(self):
        return self.patch(
            viewname='rest_api:announcement-detail', kwargs={
                'announcement_id': self._test_announcement.pk
            }, data={
                'label': TEST_ANNOUNCEMENT_LABEL_EDITED,
                'text': TEST_ANNOUNCEMENT_TEXT_EDITED
            }
        )

    def _request_announcement_edit_via_put_view(self):
        return self.put(
            viewname='rest_api:announcement-detail', kwargs={
                'announcement_id': self._test_announcement.pk
            }, data={
                'label': TEST_ANNOUNCEMENT_LABEL_EDITED,
                'text': TEST_ANNOUNCEMENT_TEXT_EDITED
            }
        )


class AnnouncementTestMixin:
    def _create_test_announcement(self):
        self._test_announcement = Announcement.objects.create(
            label=TEST_ANNOUNCEMENT_LABEL,
            text=TEST_ANNOUNCEMENT_TEXT
        )


class AnnouncementViewTestMixin:
    def _request_test_announcement_create_view(self):
        pk_list = list(
            Announcement.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='announcements:announcement_create', data={
                'label': TEST_ANNOUNCEMENT_LABEL,
                'text': TEST_ANNOUNCEMENT_TEXT
            }
        )

        try:
            self._test_announcement = Announcement.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Announcement.DoesNotExist:
            self._test_announcement = None

        return response

    def _request_test_announcement_delete_view(self):
        return self.post(
            viewname='announcements:announcement_single_delete', kwargs={
                'announcement_id': self._test_announcement.pk
            }
        )

    def _request_test_announcement_edit_view(self):
        return self.post(
            viewname='announcements:announcement_edit', kwargs={
                'announcement_id': self._test_announcement.pk
            }, data={
                'label': TEST_ANNOUNCEMENT_LABEL_EDITED,
                'text': TEST_ANNOUNCEMENT_TEXT_EDITED
            }
        )

    def _request_test_announcement_list_view(self):
        return self.get(viewname='announcements:announcement_list')



import logging

from django.apps import apps
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.document_states.classes import WorkflowAction
from mayan.apps.permissions.classes import Permission
from mayan.apps.permissions.models import Role

from .classes import ModelPermission
from .permissions import permission_acl_edit

__all__ = (
    'GrantAccessAction', 'GrantDocumentAccessAction', 'RevokeAccessAction',
    'RevokeDocumentAccessAction'
)
logger = logging.getLogger(name=__name__)


class GrantAccessAction(WorkflowAction):
    form_field_widgets = {
        'content_type': {
            'class': 'django.forms.widgets.Select', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        },
        'roles': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        },
        'permissions': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        }
    }
    form_fields = {
        'content_type': {
            'label': _('Object type'),
            'class': 'django.forms.ModelChoiceField', 'kwargs': {
                'help_text': _(
                    'Type of the object for which the access will be modified.'
                ),
                'queryset': ContentType.objects.none(),
                'required': True
            }
        }, 'object_id': {
            'label': _('Object ID'),
            'class': 'django.forms.IntegerField', 'kwargs': {
                'help_text': _(
                    'Numeric identifier of the object for which the access '
                    'will be modified.'
                ), 'required': True
            }
        }, 'roles': {
            'label': _('Roles'),
            'class': 'django.forms.ModelMultipleChoiceField', 'kwargs': {
                'help_text': _('Roles whose access will be modified.'),
                'queryset': Role.objects.all(), 'required': True
            }
        }, 'permissions': {
            'label': _('Permissions'),
            'class': 'django.forms.MultipleChoiceField', 'kwargs': {
                'help_text': _(
                    'Permissions to grant/revoke to/from the role for the '
                    'object selected above.'
                ), 'choices': (),
                'required': True
            }
        }
    }
    label = _('Grant object access')

    @classmethod
    def clean(cls, request, form_data=None):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        content_type = ContentType.objects.get(
            pk=int(
                form_data['action_data']['content_type']
            )
        )
        obj = content_type.get_object_for_this_type(
            pk=int(
                form_data['action_data']['object_id']
            )
        )

        try:
            AccessControlList.objects.check_access(
                obj=obj, permissions=(permission_acl_edit,),
                user=request.user
            )
        except Exception as exception:
            raise ValidationError(message=exception)
        else:
            return form_data

    @classmethod
    def get_form_fields(cls, *args, **kwargs):
        fields = super().get_form_fields(*args, **kwargs)

        fields['content_type']['kwargs']['queryset'] = ModelPermission.get_classes(
            as_content_type=True
        ).order_by('model')

        fields['permissions']['kwargs']['choices'] = Permission.get_choices()

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Object'), {
                    'fields': ('content_type', 'object_id')
                }
            ), (
                _('Access'), {
                    'fields': ('roles', 'permissions')
                },
            ),
        )
        return fieldsets

    def get_execute_data(self):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        content_type = ContentType.objects.get(
            pk=self.kwargs['content_type']
        )
        self.obj = content_type.get_object_for_this_type(
            pk=self.kwargs['object_id']
        )
        self.roles = Role.objects.filter(
            pk__in=self.kwargs['roles']
        )
        self.permissions = [
            Permission.get(
                pk=permission
            ) for permission in self.kwargs['permissions']
        ]

    def execute(self, context):
        self.get_execute_data()

        for role in self.roles:
            for permission in self.permissions:
                AccessControlList.objects.grant(
                    obj=self.obj, permission=permission, role=role
                )


class RevokeAccessAction(GrantAccessAction):
    label = _('Revoke object access')

    def execute(self, context):
        self.get_execute_data()

        for role in self.roles:
            for permission in self.permissions:
                AccessControlList.objects.revoke(
                    obj=self.obj, permission=permission, role=role
                )


class GrantDocumentAccessAction(WorkflowAction):
    form_field_widgets = {
        'roles': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        },
        'permissions': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        }
    }
    form_fields = {
        'roles': {
            'label': _('Roles'),
            'class': 'django.forms.ModelMultipleChoiceField', 'kwargs': {
                'help_text': _('Roles whose access will be modified.'),
                'queryset': Role.objects.all(), 'required': True
            }
        }, 'permissions': {
            'label': _('Permissions'),
            'class': 'django.forms.MultipleChoiceField', 'kwargs': {
                'help_text': _(
                    'Permissions to grant/revoke to/from the role for the '
                    'object selected above.'
                ), 'choices': (),
                'required': True
            }
        }
    }
    label = _('Grant document access')

    @classmethod
    def get_form_fields(cls, *args, **kwargs):
        fields = super().get_form_fields(*args, **kwargs)

        fields['permissions']['kwargs']['choices'] = ModelPermission.get_choices_for_class(
            klass=Document
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Access'), {
                    'fields': ('roles', 'permissions')
                },
            ),
        )
        return fieldsets

    def get_execute_data(self):
        self.roles = Role.objects.filter(
            pk__in=self.kwargs['roles']
        )
        self.permissions = [
            Permission.get(
                pk=permission
            ) for permission in self.kwargs['permissions']
        ]

    def execute(self, context):
        self.get_execute_data()

        for role in self.roles:
            for permission in self.permissions:
                AccessControlList.objects.grant(
                    obj=context['workflow_instance'].document, permission=permission, role=role
                )


class RevokeDocumentAccessAction(GrantDocumentAccessAction):
    label = _('Revoke document access')

    def execute(self, context):
        self.get_execute_data()

        for role in self.roles:
            for permission in self.permissions:
                AccessControlList.objects.revoke(
                    obj=context['workflow_instance'].document, permission=permission, role=role
                )



from django import forms
from django.utils.html import escape
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.forms import DetailForm

from .models import Key


class KeyDetailForm(DetailForm):
    def __init__(self, *args, **kwargs):
        instance = kwargs['instance']

        extra_fields = (
            {
                'field': 'key_id', 'label': _('Key ID')
            },
            {
                'func': lambda x: escape(instance.user_id),
                'label': _('User ID')
            },
            {
                'field': 'creation_date', 'label': _('Creation date'),
                'widget': forms.widgets.DateInput
            },
            {
                'func': lambda x: instance.expiration_date or _('None'),
                'label': _('Expiration date'),
                'widget': forms.widgets.DateInput
            },
            {
                'field': 'fingerprint', 'label': _('Fingerprint')
            },
            {
                'field': 'length', 'label': _('Length')
            },
            {
                'field': 'algorithm', 'label': _('Algorithm')
            },
            {
                'func': lambda x: instance.get_key_type_display(),
                'label': _('Type')
            }
        )

        kwargs['extra_fields'] = extra_fields
        super().__init__(*args, **kwargs)

    class Meta:
        fields = ()
        model = Key


class KeySearchForm(forms.Form):
    term = forms.CharField(
        label=_('Term'),
        help_text=_('Name, e-mail, key ID or key fingerprint to look for.')
    )



from django.contrib.contenttypes.models import ContentType

from mayan.apps.document_states.tests.mixins.workflow_template_state_action_mixins import WorkflowTemplateStateActionTestMixin
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.testing.tests.base import BaseTestCase

from ..workflow_actions import (
    GrantAccessAction, GrantDocumentAccessAction, RevokeAccessAction,
    RevokeDocumentAccessAction
)


class ACLActionTestCase(
    WorkflowTemplateStateActionTestMixin, BaseTestCase
):
    def test_grant_access_action(self):
        self._execute_workflow_template_state_action(
            klass=GrantAccessAction, kwargs={
                'content_type': ContentType.objects.get_for_model(
                    model=self._test_document
                ).pk,
                'object_id': self._test_document.pk,
                'roles': [self._test_case_role.pk],
                'permissions': [permission_document_view.pk]
            }
        )
        self.assertEqual(
            self._test_document.acls.count(), 1
        )
        self.assertEqual(
            list(
                self._test_document.acls.first().permissions.all()
            ), [permission_document_view.stored_permission]
        )
        self.assertEqual(
            self._test_document.acls.first().role, self._test_case_role
        )

    def test_grant_document_access_action(self):
        self._execute_workflow_template_state_action(
            klass=GrantDocumentAccessAction, kwargs={
                'roles': [self._test_case_role.pk],
                'permissions': [permission_document_view.pk]
            }
        )
        self.assertEqual(
            self._test_document.acls.count(), 1
        )
        self.assertEqual(
            list(
                self._test_document.acls.first().permissions.all()
            ), [permission_document_view.stored_permission]
        )
        self.assertEqual(
            self._test_document.acls.first().role, self._test_case_role
        )

    def test_revoke_access_action(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._execute_workflow_template_state_action(
            klass=RevokeAccessAction, kwargs={
                'content_type': ContentType.objects.get_for_model(
                    model=self._test_document
                ).pk,
                'object_id': self._test_document.pk,
                'roles': [self._test_case_role.pk],
                'permissions': [permission_document_view.pk]
            }
        )
        self.assertEqual(
            self._test_document.acls.count(), 0
        )

    def test_revoke_document_access_action(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._execute_workflow_template_state_action(
            klass=RevokeDocumentAccessAction, kwargs={
                'roles': [self._test_case_role.pk],
                'permissions': [permission_document_view.pk]
            }
        )
        self.assertEqual(
            self._test_document.acls.count(), 0
        )



from xmlrpc import client

import mayan

from .exceptions import DependenciesException
from .literals import (
    MAYAN_PYPI_NAME, MESSAGE_GREATER_THAN_SERVER, MESSAGE_NOT_LATEST,
    MESSAGE_UNKNOWN_VERSION, MESSAGE_UNEXPECTED_ERROR, MESSAGE_UP_TO_DATE,
    PYPI_URL
)
from .versions import Version


class PyPIClient:
    class ExceptionAheadOfUpstream(DependenciesException):
        """
        The installed version is more recent than the upstream version.
        """
        def __init__(self, version_local, version_server):
            self.version_local = version_local
            self.version_server = version_server
            self.message = MESSAGE_GREATER_THAN_SERVER % {
                'version_local': self.version_local,
                'version_server': self.version_server
            }

            super().__init__(self.message)

    class ExceptionNotLatestVersion(DependenciesException):
        """
        The installed version is not the latest available version.
        """
        def __init__(self, version_local, version_server):
            self.version_local = version_local
            self.version_server = version_server
            self.message = MESSAGE_NOT_LATEST % {
                'version_local': self.version_local,
                'version_server': self.version_server
            }

            super().__init__(self.message)

    class ExceptionUnknownLatestVersion(DependenciesException):
        """
        It is not possible to determine what is the latest upstream version.
        """
        def __init__(self):
            self.message = MESSAGE_UNKNOWN_VERSION

            super().__init__(self.message)

    def __init__(self):
        self.version_local = Version(version_string=mayan.__version__)

    def check_version(self):
        versions = self.get_server_versions()
        if not versions:
            raise PyPIClient.ExceptionUnknownLatestVersion
        else:
            version_local = Version(version_string=mayan.__version__)
            version_server = Version(version_string=versions[0])

            if version_local > version_server:
                raise PyPIClient.ExceptionAheadOfUpstream(
                    version_local=mayan.__version__,
                    version_server=versions[0]
                )
            elif version_local < version_server:
                raise PyPIClient.ExceptionNotLatestVersion(
                    version_local=mayan.__version__,
                    version_server=versions[0]
                )

    def check_version_verbose(self):
        try:
            self.check_version()
        except PyPIClient.ExceptionAheadOfUpstream as exception:
            message = str(exception)
        except PyPIClient.ExceptionNotLatestVersion as exception:
            message = str(exception)
        except PyPIClient.ExceptionUnknownLatestVersion as exception:
            message = str(exception)
        except Exception as exception:
            message = MESSAGE_UNEXPECTED_ERROR % {'exception': exception}
        else:
            message = MESSAGE_UP_TO_DATE % {
                'version_local': mayan.__version__
            }

        return message

    def get_server_proxy(self):
        return client.ServerProxy(uri=PYPI_URL)

    def get_server_versions(self):
        server_proxy = self.get_server_proxy()
        return server_proxy.package_releases(MAYAN_PYPI_NAME)



class ViewMixinDynamicFormBackendClass:
    def get_form_schema(self):
        return self.get_backend_class().get_form_schema(
            **self.get_form_schema_extra_kwargs()
        )

    def get_form_schema_extra_kwargs(self):
        return {}



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0015_auto_20200501_0631'),
        ('storage', '0002_auto_20200528_0826')
    ]

    operations = [
        migrations.DeleteModel(
            name='SharedUploadedFile',
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from bleach import Cleaner
from bleach.linkifier import LinkifyFilter

from mayan.apps.templating.classes import Template


class MessageBusinessLogicMixin:
    def get_label(self):
        return Template(
            template_string='{{ instance.date_time }} @{{ instance.sender_object }} "{{ instance.subject }}"'
        ).render(
            context={'instance': self}
        )
    get_label.short_description = _('Label')

    def get_rendered_body(self):
        cleaner = Cleaner(
            filters=[LinkifyFilter]
        )

        template = Template(
            template_string=cleaner.clean(text=self.body)
        )
        return template.render(
            context={'message': self}
        )

    def mark_read(self, user):
        self._event_actor = user
        self.read = True
        self.save(
            update_fields=('read',)
        )

    def mark_unread(self, user):
        self._event_actor = user
        self.read = False
        self.save(
            update_fields=('read',)
        )



from django.apps import apps
from django.db.models import Sum
from django.template.defaultfilters import filesizeformat
from django.urls import reverse_lazy
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.dashboards.classes import DashboardWidgetNumeric

from .icons import icon_file_caching
from .permissions import permission_cache_view


class DashboardWidgetFileCacheSizeAllocated(DashboardWidgetNumeric):
    icon = icon_file_caching
    label = _('Total cache allocated')
    link = reverse_lazy(viewname='file_caching:cache_list')

    def get_count(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        Cache = apps.get_model(
            app_label='file_caching', model_name='Cache'
        )

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_cache_view, user=self.request.user,
            queryset=Cache.objects.all()
        )

        total_size_allocated = queryset.aggregate(
            total_size=Sum('maximum_size')
        )['total_size'] or 0

        return format_lazy(
            '{}', filesizeformat(
                bytes_=total_size_allocated
            )
        )


class DashboardWidgetFileCacheSizeUsed(DashboardWidgetNumeric):
    icon = icon_file_caching
    label = _('Total cache used')
    link = reverse_lazy(viewname='file_caching:cache_list')

    def get_count(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        Cache = apps.get_model(
            app_label='file_caching', model_name='Cache'
        )

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_cache_view, user=self.request.user,
            queryset=Cache.objects.all()
        )

        total_size_allocated = queryset.aggregate(
            total_size=Sum('maximum_size')
        )['total_size'] or 1  # Cannot be 0 to avoid a division by zero.

        total_size_used = queryset.aggregate(
            total_size=Sum('partitions__files__file_size')
        )['total_size'] or 0

        return format_lazy(
            '{} ({:0.1f}%)', filesizeformat(
                bytes_=total_size_used
            ), total_size_used / total_size_allocated * 100
        )
